# bseen.tcl v1.4.2c (Feb 2000)
# written by Bass <bass@tclslave.net>, <bass@mars.age.psu.edu>
#
# Rewrite and conversion to a wolfpack module (Jun 2001)
#  by Tothwolf (tothwolf@concentric.net)
#
# name: bseen
# version: 1.4.4.0
# config: bseen.conf
# description: advanced seen script with nickname database and search \
#	functions.
# requires: duration isbotnick findHandOnParty isBotHand isNumber \
#	moduleData
# provides: searchSeenData addSeenData
#
# $Id: bseen.tcl,v 1.22 2001/06/06 13:28:40 tothwolf Exp $
#
# This script maintains a database of *all* nicks seen online, not just
# for handles that are listed in the userlist. Wildcard searches are also
# supported.
#
# Input is accepted from:
#   pub:  <command char>seen <query> [#chan]
#   msg:  /msg <botnick> seen <query> [#chan]
#   dcc:  .seen <query> [#chan]
#
# Queries can be in the following formats (public examples given):
#   'regular' !seen lamer; !seen lamest          | SmartSearch-enabled query
#   'limited' !seennick lamer                    | SmartSearch-bypassed query
#   'masked'  !seen *l?mer*; !seen *.lame.com; !seen *.edu #mychan
#
# Bonus feature:
#   pub:  <command char>lastspoke <nick>
#   You can use wildcard matching for <nick>.
#   Results for the first match are returned.

namespace eval bseen {

  ##
  ## Seach seen data for the given name
  ##
  ## Args:    who, where
  ## Returns: seen data if found,
  ##          -2 if no seen data loaded
  ##          -1 if error in seen data
  ##           0 if no matching data found
  ##
  proc searchSeenData {who {where ""}} {
    variable configData
    variable seendata

    if {[info exists seendata]} then {
      set who [string tolower $who]
      if {[lsearch -exact [array names seendata] $who] != -1} then {
        set data $seendata($who)
        set name [lindex $data 0]
        set time [duration [expr [unixtime] - [lindex $data 2]]]
        set output "$name ([lindex $data 1]) was last seen"
        set chan [lindex $data 4]
        if {([string compare [string tolower $where] $chan]) && \
            ([validchan $chan]) && \
            ([lsearch -exact [channel info $chan] "+secret"] != -1)} then {
          set chan "(hidden channel)"
        }
        switch -exact -- [lindex $data 3] {
          part {
            append output " parting $chan $time ago"
            set reason [lindex $data 5]
            if {[string compare "" $reason]} then {
              append output " stating \"$reason\"."
            } else {
              append output .
            }
          }
          quit {
            append output " quitting from $chan $time ago stating ([lindex $data 5])."
          }
          kick {
            append output " being kicked from $chan by [lindex $data 5] $time ago with the reason ([lindex $data 6])."
          }
          rnck {
            append output " changing nicks from [lindex $data 5] on $chan $time ago."
            if {[validchan $chan]} then {
              if {[onchan $name $chan]} then {
                append output " $name is still there."
              } else {
                append output " I don't see $name now, though."
              }
            }
          }
          nick {
            append output " changing nicks to [lindex $data 5] on $chan $time ago."
          }
          splt {
            append output " parting $chan due to a netsplit $time ago."
          }
          rejn {
            append output " rejoining $chan from a netsplit $time ago."
            if {[validchan $chan]} then {
              if {[onchan $name $chan]} then {
                append output " $name is still on $chan."
              } else {
                append output " I don't see $name on $chan now, though."
              }
            }
          }
          join {
            append output " joining $chan $time ago."
            if {[validchan $chan]} then {
              if {[onchan $name $chan]} then {
                append output " $name is still on $chan."
              } else {
                append output " I don't see $name on $chan now, though."
              }
            }
          }
          away {
            # FIXME: improve
            set reason [lindex $data 5]
            if {![string compare "" $reason]} then {
              append output " returning to the partyline on $chan $time ago."
            } else {
              append output " being marked as away ($reason) on $chan $time ago."
            }
          }
          chon {
            append output " joining the partyline $time ago."
            if {(!$configData(hideparty)) && \
                ([string compare "" [set partydata [findHandOnParty $name]]])} then {
              set partybot [lindex $partydata 1]
              set partychan [lindex $partydata 6]
              if {[string compare "" $partychan]} then {
                switch -exact -- $partychan {
                  -1 {
                    append output " $name is in the bot $partybot."
                  }
                  0 {
                    append output " $name is on the partyline in the bot ${partybot}."
                  }
                  default {
                    if {[string compare "" [set partychanname [assoc $partychan]]]} then {
                      append output " $name is on the channel \"${partychanname}\" ($partychan) in the bot ${partybot}."
                    } else {
                      append output " $name is on channel $partychan in the bot ${partybot}."
                    }
                  }
                }
              } else {
                append output " $name is in the bot ${partybot}."
              }
            } else {
              append output " I don't see $name on the partyline now, though."
            }
          }
          chof {
            append output " leaving the partyline $time ago."
            if {(!$configData(hideparty)) && \
                ([string compare "" [set partydata [findHandOnParty $name]]])} then {
              set partybot [lindex $partydata 1]
              set partychan [lindex $partydata 6]
              if {[string compare "" $partychan]} then {
                switch -exact -- $partychan {
                  -1 {
                    append output " $name is in the bot $partybot still."
                  }
                  0 {
                    append output " $name is on the partyline in the bot $partybot still."
                  }
                  default {
                    if {[string compare "" [set partychanname [assoc $partychan]]]} then {
                      append output " $name is on the channel \"${partychanname}\" ($partychan) in the bot $partybot still."
                    } else {
                      append output " $name is on channel $partychan in the bot $partybot still."
                    }
                  }
                }
              } else {
                append output " $name is in the bot $partybot still."
              }
            }
          }
          chjn {
            append output " joining the partyline on $chan $time ago."
            if {(!$configData(hideparty)) && \
                ([string compare "" [set partydata [findHandOnParty $name]]])} then {
              set partybot [lindex $partydata 1]
              set partychan [lindex $partydata 6]
              if {[string compare "" $partychan]} then {
                switch -exact -- $partychan {
                  -1 {
                    append output " $name is in the bot $partybot."
                  }
                  0 {
                    append output " $name is on the partyline in the bot ${partybot}."
                  }
                  default {
                    if {[string compare "" [set partychanname [assoc $partychan]]]} then {
                      append output " $name is on the channel \"${partychanname}\" ($partychan) in the bot ${partybot}."
                    } else {
                      append output " $name is on channel $partychan in the bot ${partybot}."
                    }
                  }
                }
              } else {
                append output " $name is in the bot ${partybot}."
              }
            } else {
              append output " I don't see $name on the partyline now, though."
            }
          }
          chpt {
            append output " leaving the partyline from $chan $time ago."
            if {(!$configData(hideparty)) && \
                ([string compare "" [set partydata [findHandOnParty $name]]])} then {
              set partybot [lindex $partydata 1]
              set partychan [lindex $partydata 6]
              if {[string compare "" $partychan]} then {
                switch -exact -- $partychan {
                  -1 {
                    append output " $name is in the bot $partybot still."
                  }
                  0 {
                    append output " $name is on the partyline in the bot $partybot still."
                  }
                  default {
                    if {[string compare "" [set partychanname [assoc $partychan]]]} then {
                      append output " $name is on the channel \"${partychanname}\" ($partychan) in the bot $partybot still."
                    } else {
                      append output " $name is on channel $partychan in the bot $partybot still."
                    }
                  }
                }
              } else {
                append output " $name is in the bot $partybot still."
              }
            }
          }
          default {
            return -1
          }
        }
        return $output
      } else {
        return 0
      }
    } else {
      return -2
    }
  }

  ##
  ## Add data to seen array
  ##
  ## Args:    nick, data
  ## Returns: nothing
  ##
  proc addSeenData {nick data} {
    variable SeenConfig
    variable seendata

    set chan [string tolower [lindex $data 4]]
    if {([lsearch -exact $SeenConfig(no_log) $chan] == -1) && \
        ((![string compare "" $SeenConfig(log_only)]) || \
         ([lsearch -exact $SeenConfig(log_only) $chan] != -1))} then {
      set seendata([string tolower $nick]) "$nick $data"
    }
    return
  }

  ##
  ## Add data to seen array: join
  ##
  ## Args:    nick, user@host, handle, channel
  ## Returns: 0
  ##
  proc join:seen {nick uhost hand chan} {
    addSeenData $nick [list $uhost [unixtime] join $chan]
    return 0
  }

  ##
  ## Add data to seen array: part
  ##
  ## Args:    nick, user@host, handle, channel, reason
  ## Returns: 0
  ##
  proc part:seen {nick uhost hand chan {reason ""}} {
    addSeenData $nick [list $uhost [unixtime] part $chan $reason]
    return 0
  }

  ##
  ## Add data to seen array: sign
  ##
  ## Args:    nick, user@host, handle, channel, reason
  ## Returns: 0
  ##
  proc sign:seen {nick uhost hand chan reason} {
    addSeenData $nick [list $uhost [unixtime] quit $chan $reason]
    return 0
  }

  ##
  ## Add data to seen array: kick
  ##
  ## Args:    nick, user@host, handle, channel, kicked nick, reason
  ## Returns: 0
  ##
  proc kick:seen {nick uhost hand chan knick reason} {
    addSeenData $knick [list [getchanhost $knick $chan] [unixtime] kick $chan $nick $reason]
    return 0
  }

  ##
  ## Add data to seen array: nick
  ##
  ## Args:    nick, user@host, handle, channel, new nick
  ## Returns: 0
  ##
  proc nick:seen {nick uhost hand chan newnick} {
    set time [unixtime]
    addSeenData $nick [list $uhost [expr $time - 1] nick $chan $newnick]
    addSeenData $newnick [list $uhost $time rnck $chan $nick]
    return 0
  }

  ##
  ## Add data to seen array: splt
  ##
  ## Args:    nick, user@host, handle, channel
  ## Returns: 0
  ##
  proc splt:seen {nick uhost hand chan} {
    addSeenData $nick [list $uhost [unixtime] splt $chan]
    return 0
  }

  ##
  ## Add data to seen array: rejn
  ##
  ## Args:    nick, user@host, handle, channel
  ## Returns: 0
  ##
  proc rejn:seen {nick uhost hand chan} {
    addSeenData $nick [list $uhost [unixtime] rejn $chan]
    return 0
  }

  ##
  ## Add data to seen array: away
  ##
  ## Args:    bot, idx, text
  ## Returns: 0
  ##
  proc away:seen {bot idx text} {
    if {[isBotHand $bot]} then {
      set hand [idx2hand $idx]
      # FIXME: searchSeenData and string trim are hacks
      set result [searchSeenData [string tolower $hand]]
      if {![isNumber $result]} then {
        addSeenData $hand [list [string trim [lindex $result 1] ()] [unixtime] away $bot $text]
      }
    }
    return 0
  }

  ##
  ## Add data to seen array: chon
  ##
  ## Args:    handle, idx
  ## Returns: 0
  ##
  proc chon:seen {hand idx} {
    foreach {dccidx dcchand uhost type extra time} [dcclist CHAT] {
      if {$idx == $dccidx} then {
        addSeenData $hand [list $uhost [unixtime] chon]
        break
      }
    }
    return 0
  }

  ##
  ## Add data to seen array: chof
  ##
  ## Args:    handle, idx
  ## Returns: 0
  ##
  proc chof:seen {hand idx} {
    # FIXME: searchSeenData and string trim are hacks
    set result [searchSeenData [string tolower $hand]]
    if {![isNumber $result]} then {
      addSeenData $hand [list [string trim [lindex $result 1] ()] [unixtime] chof]
    }
    return 0
  }

  ##
  ## Add data to seen array: chjn
  ##
  ## Args:    bot, handle, channel number, flag, idx, from
  ## Returns: 0
  ##
  proc chjn:seen {bot hand channum flag idx from} {
    addSeenData $hand [list [string trimleft $from ~] [unixtime] chjn $bot]
    return 0
  }

  ##
  ## Add data to seen array: chpt
  ##
  ## Args:    bot, handle, idx, channum
  ## Returns: 0
  ##
  proc chpt:seen {bot hand idx channum} {
    # FIXME: searchSeenData and string trim are hacks
    set result [searchSeenData [string tolower $hand]]
    if {![isNumber $result]} then {
      addSeenData $hand [list [string trim [lindex $result 1] ()] [unixtime] chpt $bot]
    }
    return 0
  }

  ##
  ## Init bot idle time for channels: join
  ##
  ## Args:    nick, user@host, handle, channel
  ## Returns: 0
  ##
  proc join:botidle {nick uhost hand chan} {
    variable SeenIdle

    if {[isbotnick $nick]} then {
      set SeenIdle([string tolower $chan]) [unixtime]
    }
    return 0
  }

  ##
  ## Save seen data every hour
  ##
  ## Args:    minute, hour, day, month, year
  ## Returns: 0
  ##
  proc time:hourly {minute hour day month year} {
    variable NamespaceTail

    moduleData save $NamespaceTail 1
    return 0
  }

################################################################################

# FIXME: finish rewrite

if {0} then {

proc bs_unseen {hand idx arg} {
  variable bs_list

  set tot 0
  set chan [string tolower [lindex $arg 0]]
  set id [array startsearch bs_list]
  while {[array anymore bs_list $id]} {
    set item [array nextelement bs_list $id]
    if {$chan == [string tolower [lindex $bs_list($item) 4]]} then {
      incr tot
      lappend remlist $item
    }
  }
  array donesearch bs_list $id
  if {$tot > 0} then {
    foreach item $remlist {
      unset bs_list($item)
    }
  }
  putlog "$hand removed $chan from the bseen database.  $tot entries removed."
  putdcc $idx "$chan successfully removed.  $tot entries deleted from the bseen database."
}


proc bs_botsearch {from cmd arg} {
  global botnick

  set command [lindex $arg 0]
  set target [lindex $arg 1]
  set nick [lindex $arg 2]
  set search [join [lrange $arg 3 end]]
  if {[string match *\\\** $search]} then {
    set output [bs_seenmask bot $nick $search]
    if {$output != "No matches were found." && ![string match "I'm not on *" $output]} then {
      putbot $from "bs_botsearch_reply $command \{$target\} {$nick, $botnick says:  $output}"
    }
  } else {
    set output [bs_output bot $nick [lindex $search 0] 0]
    if {$output != 0 && [lrange [split $output] 1 4] != "I don't remember seeing"} then {
      putbot $from "bs_botsearch_reply $command \{$target\} {$nick, $botnick says: $output}"
    }
  }
}

if {[info exists configData(bot_delay)]} then {
  unset configData(bot_delay)
}


proc bs_botsearch_reply {from cmd arg} {
  variable configData

  if {[lindex [lindex $arg 2] 5] == "not" || [lindex [lindex $arg 2] 4] == "not"} then {
    return
  }
  if {![info exists configData(bot_delay)]} then {
    set configData(bot_delay) on
    utimer 10 {if {[info exists configData(bot_delay)]} {unset configData(bot_delay)}}
    if {![lindex $arg 0]} then {
      putdcc [lindex $arg 1] "[join [lindex $arg 2]]"
    } else {
      puthelp "[lindex $arg 1] :[join [lindex $arg 2]]"
    }
  }
}


proc bs_dccreq1 {hand idx arg} {
  bs_dccreq $hand $idx $arg 0
}


proc bs_dccreq2 {hand idx arg} {
  bs_dccreq $hand $idx $arg 1
}


proc bs_dccreq {hand idx arg no} {
  variable configData

  if {[string match *\\\** [lindex $arg 0]]} then {
    set output [bs_seenmask dcc $hand $arg]
    if {$output == "No matches were found."} then {
      putallbots "bs_botsearch 0 $idx $hand $arg"
    }
    if {[string match "I'm not on *" $output]} then {
      putallbots "bs_botsearch 0 $idx $hand $arg"
    }
    putdcc $idx $output
    return $configData(logqueries)
  }
  set search [lindex $arg 0]
  set output [bs_output dcc $hand $search $no]
  if {$output == 0} then {
    return 0
  }
  if {[lrange [split $output] 1 4] == "I don't remember seeing"} then {
    putallbots "bs_botsearch 0 $idx $hand $arg"
  }
  putdcc $idx "$output"
  return $configData(logqueries)
}


proc bs_msgreq1 {nick uhost hand arg} {
  bs_msgreq $nick $uhost $hand $arg 0
}


proc bs_msgreq2 {nick uhost hand arg} {
  bs_msgreq $nick $uhost $hand $arg 1
}


proc bs_msgreq {nick uhost hand arg no} {
  variable configData

  if {[bs_flood $nick $uhost]} then {
    return 0
  }
  if {[string match *\\\** [lindex $arg 0]]} then {
    set output [bs_seenmask msg $nick $arg]
    if {$output == "No matches were found."} then {
      putallbots "bs_botsearch 1 \{notice $nick\} $nick $arg"
    }
    if {[string match "I'm not on *" $output]} then {
      putallbots "bs_botsearch 1 \{notice $nick\} $nick $arg"
    }
    puthelp "notice $nick :$output"
    return $configData(logqueries)
  }
  set search [lindex $arg 0]
  set output [bs_output $search $nick $search $no]
  if {$output == 0} then {
    return 0
  }
  if {[lrange [split $output] 1 4] == "I don't remember seeing"} then {
    putallbots "bs_botsearch 1 \{notice $nick\} $nick $arg"
  }
  puthelp "notice $nick :$output"
  return $configData(logqueries)
}


proc bs_pubreq1 {nick uhost hand chan arg} {
  bs_pubreq $nick $uhost $hand $chan $arg 0
}


proc bs_pubreq2 {nick uhost hand chan arg} {
  bs_pubreq $nick $uhost $hand $chan $arg 1
}


proc bs_pubreq {nick uhost hand chan arg no} {
  global botnick
  variable configData

  if {[bs_flood $nick $uhost]} then {
    return 0
  }
  set i 0
  if {[lsearch -exact $configData(no_pub) [string tolower $chan]] >= 0} then {
    return 0
  }
  if {$configData(log_only) != "" && [lsearch -exact $configData(log_only) [string tolower $chan]] == -1} then {
    return 0
  }
  if {[lsearch -exact $configData(quiet_chan) [string tolower $chan]] >= 0} then {
    set target "notice $nick"
  } else {
    set target "privmsg $chan"
  }
  if {[string match *\\\** [lindex $arg 0]]} then {
    set output [bs_seenmask $chan $hand $arg]
    if {$output == "No matches were found."} then {
      putallbots "bs_botsearch 1 \{$target\} $nick $arg"
    }
    if {[string match "I'm not on *" $output]} then {
      putallbots "bs_botsearch 1 \{$target\} $nick $arg"
    }
    puthelp "$target :$output"
    return $configData(logqueries)
  }
  set data [string trimright [lindex $arg 0] ?!.,]
  if {[string tolower $nick] == [string tolower $data]} then {
    puthelp "$target :$nick, go look in a mirror."
    return $configData(logqueries)
  }
  if {[string tolower $data] == [string tolower $botnick]} then {
    puthelp "$target :$nick, I'm right here.  Quit wasting my time!"
    return $configData(logqueries)
  }
  if {[onchan $data $chan]} then {
    puthelp "$target :$nick, $data is right here!"
    return $configData(logqueries)
  }
  set output [bs_output $chan $nick $data $no]
  if {$output == 0} then {
    return 0
  }
  if {[lrange [split $output] 1 4] == "I don't remember seeing"} then {
    putallbots "bs_botsearch 1 \{$target\} $nick $arg"
  }
  puthelp "$target :$output"
  return $configData(logqueries)
}


proc bs_output {chan nick data no} {
  global version botnick
  variable configData
  variable bs_list

  set data [string tolower [string trimright [lindex $data 0] ?!.,]]
  if {$data == ""} then {
    return 0
  }
  if {[string tolower $nick] == $data} then {
    return [concat $nick, go look in a mirror.]
  }
  if {$data == [string tolower $botnick]} then {
    return [concat $nick, I'm right here. Quit wasting my time!]
  }
  if {[string length $data] > $configData(nicksize)} then {
    return 0
  }
  if {$configData(smartsearch) != 1} then {
    set no 1
  }
  if {$no == 0} then {
    set matches ""
    set hand ""
    set addy ""
    if {[lsearch -exact [array names bs_list] $data] != "-1"} then {
      set addy [lindex $bs_list([string tolower $data]) 1]
      set hand [finduser $addy]
      foreach item [bs_seenmask dcc ? [maskhost $addy]] {
        if {[lsearch -exact $matches $item] == -1} then {
          set matches "$matches $item"
        }
      }
    }
    if {[validuser $data]} then {
      set hand $data
    }
    if {$hand != "*" && $hand != ""} then {
      if {[string trimleft [lindex $version 1] 0]>1030000} then {
        set hosts [getuser $hand hosts]
      } else {
        set hosts [gethosts $hand]
      }
      foreach addr $hosts {
        foreach item [string tolower [bs_seenmask dcc ? $addr]] {
          if {[lsearch -exact [string tolower $matches] [string tolower $item]] == -1} then {
            set matches [concat $matches $item]
          }
        }
      }
    }
    if {$matches != ""} then {
      set matches [string trimleft $matches " "]
      set len [llength $matches]
      if {$len == 1} then {
        return [bs_search $chan [lindex $matches 0]]
      }
      if {$len > 99} then {
        return [concat I found $len matches to your query\; please refine it to see any output.]
      }
      set matches [bs_sort $matches]
      set key [lindex $matches 0]
      if {[string tolower $key] == [string tolower $data]} then {
        return [bs_search $chan $key]
      }
      if {$len <= 5} then {
        set output [concat I found $len matches to your query (sorted): [join $matches].]
        set output [concat $output [bs_search $chan $key]]
        return $output
      } else {
        set output [concat I found $len matches to your query. Here are the 5 most recent (sorted): [join [lrange $matches 0 4]].]
        set output [concat $output [bs_search $chan $key]]
        return $output
      }
    }
  }
  set temp [bs_search $chan $data]
  if {$temp != 0} then {
    return $temp
  } else {
    #$data not found in $bs_list, so search userfile
    if {![validuser $data] || [string trimleft [lindex $version 1] 0]<1030000} then {
      return "$nick, I don't remember seeing $data."
    } else {
      set seen [getuser $data laston]
      if {[getuser $data laston] == ""} then {
        return "$nick, I don't remember seeing $data."
      }
      if {($chan != [lindex $seen 1] || $chan == "bot" || $chan == "msg" || $chan == "dcc") && [validchan [lindex $seen 1]] && [lindex [channel info [lindex $seen 1]] 23] == "+secret"} then {
        set chan "-secret-"
      } else {
        set chan [lindex $seen 1]
      }
      return [concat $nick, $data was last seen on $chan [duration [lindex $seen 0]] ago.]
    }
  }
}


proc bs_lsortcmd {a b} {
  variable bs_list

  set a [lindex $bs_list([string tolower $a]) 2]
  set b [lindex $bs_list([string tolower $b]) 2]

  return [string compare $a $b]
}


proc bs_trim {min h d m y} {
  variable configData
  variable bs_list

  if {![info exists bs_list] || ![array exists bs_list]} then {
    return
  }
  set list [array names bs_list]
  set range [expr [llength $list] - $configData(dbmax) - 1]
  if {$range < 0} then {
    return
  }
  set list [lsort -increasing -command bs_lsortcmd $list]
  foreach item [lrange $list 0 $range] {
    unset bs_list($item)
  }
}


proc bs_seenmask {ch nick arg} {
  variable configData
  variable bs_list

  set matches ""
  set temp ""
  set i 0
  set chan [lindex $arg 1]
  if {$chan != "" && [string trimleft $chan #] != $chan} then {
    if {![validchan $chan]} then {
      return "I'm not on $chan."
    } else {
      set chan [string tolower $chan]
    }
  } else {
    set $chan ""
  }
  if {![info exists bs_list]} then {
    return "No matches were found."
  }
  set data [string tolower [lindex $arg 0]]

  #bugfix: unnecessarily complex masks essentially freeze the bot
  set maskfix 1
  while {$maskfix} {
    set mark 1
    if {[regsub -all -- \\?\\? $data ? data]} then {
      set mark 0
    }
    if {[regsub -all -- \\*\\* $data * data]} then {
      set mark 0
    }
    if {[regsub -all -- \\*\\? $data * data]} then {
      set mark 0
    }
    if {[regsub -all -- \\?\\* $data * data]} then {
      set mark 0
    }
    if {$mark} then {
      break
    }
  }

  set id [array startsearch bs_list]
  while {[array anymore bs_list $id]} {
    set item [array nextelement bs_list $id]
    if {$item == ""} then {
      continue
    }
    set i 0
    set temp ""
    set match [lindex $bs_list($item) 0]
    set addy [lindex $bs_list($item) 1]
    if {[string match $data $item![string tolower $addy]]} then {
      if {$chan != ""} then {
        if {[string match $chan [string tolower [lindex $bs_list($item) 4]]]} then {
          set matches [concat $matches $match]
        }
      } else {
        set matches [concat $matches $match]
      }
    }
  }
  array donesearch bs_list $id
  set matches [string trim $matches " "]
  if {$nick == "?"} then {
    return $matches
  }
  set len [llength $matches]
  if {$len == 0} then {
    return "No matches were found."
  }
  if {$len == 1} then {
    return [bs_output $ch $nick $matches 1]
  }
  if {$len > 99} then {
    return "I found $len matches to your query; please refine it to see any output."
  }
  set matches [bs_sort $matches]
  if {$len <= 5} then {
    set output [concat I found $len matches to your query (sorted): [join $matches].]
  } else {
    set output "I found $len matches to your query.  Here are the 5 most recent (sorted): [join [lrange $matches 0 4]]."
  }
  return [concat $output [bs_output $ch $nick [lindex [split $matches] 0] 1]]
}


proc bs_sort {data} {
  variable bs_list

  set data [join [lsort -decreasing -command bs_lsortcmd $data]]
  return $data
}


proc bs_dccstats {hand idx arg} {
  putdcc $idx "[bs_stats]"
  return 1
}


proc bs_pubstats {nick uhost hand chan arg} {
  variable configData

  if {[bs_flood $nick $uhost] || [lsearch -exact $configData(no_pub) [string tolower $chan]] >= 0 ||($configData(log_only) != "" && [lsearch -exact $configData(log_only) [string tolower $chan]] == -1)} then {
    return 0
  }
  if {[lsearch -exact $configData(quiet_chan) [string tolower $chan]] >= 0} then {
    set target "notice $nick"
  } else {
    set target "privmsg $chan"
  }
  puthelp "$target :[bs_stats]"
  return 1
}


proc bs_msgstats {nick uhost hand arg} {
  variable configData

  if {[bs_flood $nick $uhost]} then {
    return 0
  }
  puthelp "notice $nick :[bs_stats]"
  return $configData(logqueries)
}


proc bs_stats {} {
  variable configData
  variable bs_list

  set id [array startsearch bs_list]
  set bs_record [unixtime]
  set totalm 0
  set temp ""
  while {[array anymore bs_list $id]} {
    set item [array nextelement bs_list $id]
    set tok [lindex $bs_list($item) 2]
    if {$tok == ""} then {
      putlog "Damaged seen record: $item"
      continue
    }
    if {[lindex $bs_list($item) 2] < $bs_record} then {
      set bs_record [lindex $bs_list($item) 2]
      set name $item
    }
    set addy [string tolower [maskhost [lindex $bs_list($item) 1]]]
    if {[lsearch -exact $temp $addy] == -1} then {
      incr totalm
      lappend temp $addy
    }
  }
  array donesearch bs_list $id
  return "Currently I am tracking [array size bs_list]/$configData(dbmax) nicks, which comprise $totalm unique uhosts.  The oldest record is [lindex $bs_list($name) 0]'s, which is from [duration $bs_record] ago."
}


proc bs_dccchanstats {hand idx arg} {
  if {$arg == ""} then {
    set arg [console $idx]
  }
  if {[lindex $arg 0] == "*"} then {
    putdcc $idx "$hand, chanstats requires a channel arg, or a valid console channel."
    return 1
  }
  putdcc $idx "[bs_chanstats [lindex $arg 0]]"
  return 1
}


proc bs_pubchanstats {nick uhost hand chan arg} {
  variable configData

  set chan [string tolower $chan]
  if {[bs_flood $nick $uhost] || [lsearch -exact $configData(no_pub) $chan] >= 0 || ($configData(log_only) != "" && [lsearch -exact $configData(log_only) [string tolower $chan]] == -1)} then {
    return 0
  }
  if {[lsearch -exact $configData(quiet_chan) $chan] >= 0} then {
    set target "notice $nick"
  } else {
    set target "privmsg $chan"
  }
  if {[lindex $arg 0] != ""} then {
    set chan [lindex $arg 0]
  }
  puthelp "$target :[bs_chanstats $chan]"
  return $configData(logqueries)
}


proc bs_msgchanstats {nick uhost hand arg} {
  variable configData

  if {[bs_flood $nick $uhost]} then {
    return 0
  }
  puthelp "notice $nick :[bs_chanstats [lindex $arg 0]]"
  return $configData(logqueries)
}


proc bs_chanstats {chan} {
  variable bs_list

  set chan [string tolower $chan]
  if {![validchan $chan]} then {
    return "I'm not on $chan."
  }
  set id [array startsearch bs_list]
  set bs_record [unixtime]
  set totalc 0
  set totalm 0
  set temp ""
  while {[array anymore bs_list $id]} {
    set item [array nextelement bs_list $id]
    set time [lindex $bs_list($item) 2]
    if {$time == ""} then {
      continue
    }
    if {$chan == [string tolower [lindex $bs_list($item) 4]]} then {
      if {$time < $bs_record} then {
        set bs_record $time
      }
      incr totalc
      set addy [string tolower [maskhost [lindex $bs_list($item) 1]]]
      if {[lsearch -exact $temp $addy] == -1} then {
        incr totalm
        lappend temp $addy
      }
    }
  }
  array donesearch bs_list $id
  set total [array size bs_list]
  return "$chan is the source of [expr {100*$totalc/$total}]% ($totalc/$total) of the seen database entries.  In $chan, there were a total of $totalm unique uhosts seen in the last [duration $bs_record]."
}



proc lastspoke {nick uhost hand chan arg} {
  global botnick
  variable configData
  variable bs_botidle

  set chan [string tolower $chan]
  if {[bs_flood $nick $uhost] || [lsearch -exact $configData(no_pub) $chan] >= 0 || ($configData(log_only) != "" && [lsearch -exact $configData(log_only) $chan] == -1)} then {
    return 0
  }
  if {[lsearch -exact $configData(quiet_chan) $chan] >= 0} then {
    set target "notice $nick"
  } else {
    set target "privmsg $chan"
  }
  set data [lindex $arg 0]
  set ldata [string tolower $data]
  if {[string match *\** $data]} then {
    set chanlist [string tolower [chanlist $chan]]
    if {[lsearch -glob $chanlist $ldata] > -1} then {
      set data [lindex [chanlist $chan] [lsearch -glob $chanlist $ldata]]
    }
  }
  if {[onchan $data $chan]} then {
    if {$ldata == [string tolower $botnick]} then {
      puthelp "$target :$nick, must you waste my time?"
      return 1
    }
    set time [getchanidle $data $chan]
    set bottime [expr ([unixtime] - $bs_botidle($chan))/60]
    if {$time < $bottime} then {
      if {$time > 0} then {
        set diftime [duration [expr [unixtime] - $time*60 -15]]
      } else {
        set diftime "less than a minute"
      }
      puthelp "$target :$data last uttered a word on $chan $diftime ago."
    } else {
      set diftime [duration $bs_botidle($chan)]
      puthelp "$target :$data hasn't uttered a word since I joined $chan $diftime ago."
    }
  }
  return 1
}



proc bs_help_msg_seen {nick uhost hand arg} {
  if {[bs_flood $nick $uhost]} then {
    return 0
  }
  puthelp "notice $nick :###  seen <query> \[chan\]"
  puthelp "notice $nick :   Queries can be in the following formats:"
  puthelp "notice $nick :     'regular':  seen lamer; seen lamest "
  puthelp "notice $nick :     'masked':   seen *l?mer*; seen *.lame.com; seen *.edu #mychan"
  return 0
}


proc bs_help_msg_chanstats {nick uhost hand arg} {
  if {[bs_flood $nick $uhost]} then {
    return 0
  }
  puthelp "notice $nick :###  chanstats <chan>"
  puthelp "notice $nick :   Returns the usage statistics of #chan in the seen database."
  return 0
}


proc bs_help_msg_seenstats {nick uhost hand arg} {
  if {[bs_flood $nick $uhost]} then {
    return 0
  }
  puthelp "notice $nick :###  seenstats"
  puthelp "notice $nick :   Returns the status of the bseen database."
  return 0
}


proc bs_help_dcc {hand idx arg} {
  switch -exact -- $arg {
    seen {
      putdcc $idx "###  seen <query> \[chan\]"
      putdcc $idx "   Queries can be in the following formats:"
      putdcc $idx "     'regular':  seen lamer; seen lamest "
      putdcc $idx "     'masked':   seen *l?mer*; seen *.lame.com; seen *.edu #mychan"
    }
    seennick {
      putdcc $idx "###  seen <nick>"
    }
    chanstats {
      putdcc $idx "###  chanstats <chan>"
      putdcc $idx "   Returns the usage statistics of #chan in the seen database."
    }
    seenstats {
      putdcc $idx "###  seenstats"
      putdcc $idx "   Returns the status of the bseen database."
    }
    unseen {
      if {[matchattr $hand n]} then {
        putdcc $idx "###  unseen <chan>"
        putdcc $idx "   Deletes all <chan> entries from the bseen database."
      }
    }
    default {
      *dcc:help $hand $idx $arg
      return 0
    }
  }
  return 1
}

} ;# if 0

################################################################################

  ##
  ## Trace configuration changes
  ##
  ## Args:    name1, name2, operation
  ## Returns: nothing
  ##
  proc TraceConfigData {name1 name2 op} {
    variable configData
    variable oldConfigData

    if {[string compare $oldConfigData($name2) $configData($name2)]} then {
      switch -exact -- $name2 {
        no_log -
        log_only {
          # FIXME: messy hack
          set SeenConfig($name2) [string tolower $configData($name2)]
        }
      }
      set oldConfigData($name2) $configData($name2)
    }
    return
  }

  ##
  ## Trace and respond to variable changes
  ##
  ## Args:    action {variable|vdelete}
  ## Returns: nothing
  ##
  proc TraceTable {action} {
    variable NamespaceCurrent
    variable configData

    trace $action configData w ${NamespaceCurrent}::TraceConfigData
    return
  }

  ##
  ## Init module specific things at load time
  ##
  ## Args:    action {load|unload}
  ## Returns: nothing
  ##
  proc InitTable {action} {
    variable NamespaceCurrent
    variable NamespaceTail
    variable configData
    variable oldConfigData
    variable SeenConfig
    variable SeenIdle

    switch -exact -- $action {
      load {
        if {![info exists NamespaceCurrent]} then {
          set NamespaceCurrent [namespace current]
        }
        if {![info exists NamesapceTail]} then {
          set NamespaceTail [namespace tail $NamespaceCurrent]
        }
        foreach {name data} [array get configData] {
          if {![info exists oldConfigData($name)]} then {
            set oldConfigData($name) $data
          }
        }
        # FIXME: this is a quick hack
        foreach i [array names configData] {
          set SeenConfig($i) [string tolower $configData($i)]
        }
        # FIXME: should we do this elsewhere/differently?
        if {!$configData(hideparty)} then {
          if {![string compare "" [info commands assoc]]} then {
            loadmodule assoc
          }
        }
        foreach chan [string tolower [channels]] {
          if {![info exists SeenIdle($chan)]} then {
            set SeenIdle($chan) [clock seconds]
          }
        }
      }
    }
    return
  }

  ##
  ## Bind procs to commands and events
  ##
  ## Args:    action {bind|unbind}, what {type|*}, args {...}
  ## Returns: nothing
  ##
  proc BindTable {action what args} {
    variable NamespaceCurrent

    if {![string compare * $what]} then {
      set what "join part sign kick nick splt rejn away chon chof chjn chpt pub msg msgm dcc bot time"
    }
    foreach type $what {
      switch -exact -- $type {
        join {
          $action join -|- * ${NamespaceCurrent}::join:seen
          $action join -|- * ${NamespaceCurrent}::join:botidle
        }
        part {
          $action part -|- * ${NamespaceCurrent}::part:seen
        }
        sign {
          $action sign -|- * ${NamespaceCurrent}::sign:seen
        }
        kick {
          $action kick -|- * ${NamespaceCurrent}::kick:seen
        }
        nick {
          $action nick -|- * ${NamespaceCurrent}::nick:seen
        }
        splt {
          $action splt -|- * ${NamespaceCurrent}::splt:seen
        }
        rejn {
          $action rejn -|- * ${NamespaceCurrent}::rejn:seen
        }
        away {
          $action away -|- * ${NamespaceCurrent}::away:seen
        }
        chon {
          $action chon -|- * ${NamespaceCurrent}::chon:seen
        }
        chof {
          $action chof -|- * ${NamespaceCurrent}::chof:seen
        }
        chjn {
          $action chjn -|- * ${NamespaceCurrent}::chjn:seen
        }
        chpt {
          $action chpt -|- * ${NamespaceCurrent}::chpt:seen
        }
        pub {
#          set char [lindex $args 0]
#          if {![string compare "" $char]} then {
#            set char [commandChar pub]
#          }
#          $action pub  -|- ${char}chanstats bs_pubchanstats
#          $action pub  -|- ${char}lastspoke lastspoke
#          $action pub  -|- ${char}seen bs_pubreq1
#          $action pub  -|- ${char}seennick bs_pubreq2
#          $action pub  -|- ${char}seenstats bs_pubstats
        }
        msg {
#          $action msg  -|- chanstats bs_msgchanstats
#          $action msg  -|- seen bs_msgreq1
#          $action msg  -|- seennick bs_msgreq2
#          $action msg  -|- seenstats bs_msgstats
        }
        msgm {
#          $action msgm -|- "help chanstats" bs_help_msg_chanstats
#          $action msgm -|- "help seen" bs_help_msg_seen
#          $action msgm -|- "help seenstats" bs_help_msg_seenstats
        }
        dcc {
#          $action dcc  -|- chanstats bs_dccchanstats
#          $action dcc  -|- help bs_help_dcc
#          $action dcc  -|- seen bs_dccreq1
#          $action dcc  -|- seennick bs_dccreq2
#          $action dcc  -|- seenstats bs_dccstats
#          $action dcc  n|- unseen bs_unseen
        }
        bot {
#          $action bot  -|- bs_botsearch bs_botsearch
#          $action bot  -|- bs_botsearch_reply bs_botsearch_reply
        }
        time {
          $action time -|- "00 * * * *" ${NamespaceCurrent}::time:hourly
#          $action time -|- "*1 * * * *" bs_trim
        }
      }
    }
    return
  }

  # Configuration data defaults
  array set configDataDefaults {
    hideparty 0
    dbmax 4000
    nicksize 9
    no_pub ""
    quiet_chan ""
    no_log ""
    log_only ""
    smartsearch 1
    logqueries 1
  }

  # Configuration data descriptions
  array set configDataDesc {
    hideparty "\[0/1\] switch to hide partyline channels"
    dbmax "\[number\] database record limit"
    nicksize "\[number\] maximum nickname length"
    no_pub "\[list\] list of channels you *don't* want the bot to post public replies to"
    quiet_chan "\[list\] list of channels you want replies to requests sent to the person who made the query via notice"
    no_log "\[list\] list of channels you *don't* want the bot to log data on"
    log_only "\[list\] list of channels you *only* want the bot to log data on, the opposite of no_log"
    smartsearch "\[0/1\] SmartSearch ensures that the most accurate and current results are returned for nick queries"
    logqueries "\[0/1\] log DCC/MSG/PUB queries"
  }

  # Set datafile information
  array set dataOptions {
    seendata "array bseen.data seen"
  }

} ;# namespace bseen

return
