###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2001  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: compat.tcl,v 1.3 2001/06/07 08:19:26 tothwolf Exp $
##

# name: compat
# version: 0.6.0.0
# description: Compatibility functions for older versions of eggdrop that \
#	allow modules the ability to use new commands, but still remain \
#	backwards compatible with older eggdrop releases.
# provides: erasenotes listnotes botonchan putquick botisvoice islinked \
#	isbotnick duration putkick

namespace eval compat {

  ##
  ## erasenotes
  ##   added: 1.3.15
  ##
  #  erasenotes <user> <numberlist>
  #    erases some or all stored notes for a user
  #    returns: -1 if no such user, -2 if notefile failure, 0 if
  #             no such note, or number of erased notes.
  #             'erasenote mynick -' erase all notes for mynick.
  ##
  # FIXME: finish this proc
  proc compat:erasenotes {args} {
    global notefile

    BadArgs $args 2 2 "erasenotes handle noteslist#"
    set user [lindex $args 0]
    set notes [lidnex $args 1]
    if {![validuser $user]} then {
      return -1
    }
    if {(![info exists notefile]) || (![file exists $notefile]) || \
        (![file isfile $notefile]) || (![file writable $notefile])} then {
      return -2
    }
    if {[catch {set fd [open $notefile r]}]} then {
      return -2
    } else {
      while {![eof $fd]} {
        lappend notes [gets $fd]
      }
      close $fd

      # from to unixtime body
      # erase matching notes from $notes
      # remove last to first
      # set notes [lreplace $notes $index $index]
      # '-' removes all for that user
      # return # removed, or 0 if no match

      return ""

#      if {[catch {set fd [open $notefile w]}]} then {
#        return -2
#      } else {
#        foreach note $notes {
#          puts $fd $note
#        }
#        close $fd
#      }
    }
  }

  ##
  ## listnotes
  ##   added: 1.3.15
  ##
  #  listnotes <user> <numberlist>
  #    lists existing notes according to the numberlist (ex: "2-4;8;16-")
  #    returns: -1 if no such user, -2 if notefile failure, 0 if no
  #             such note, list of existing notes.
  ##
  # FIXME: finish this proc
  proc compat:listnotes {args} {
    global notefile

    BadArgs $args 2 2 "listnotes handle noteslist#"
    set user [lindex $args 0]
    set notes [lidnex $args 1]
    if {![validuser $user]} then {
      return -1
    }
    if {(![info exists notefile]) || (![file exists $notefile]) || \
        (![file isfile $notefile]) || (![file readable $notefile])} then {
      return -2
    }
    if {[catch {set fd [open $notefile r]}]} then {
      return -2
    } else {
      while {![eof $fd]} {
        lappend notes [gets $fd]
      }
      close $fd

      # from to unixtime body
      # list matching notes in $notes
      # '-' lists all notes
      # 1 2 3 etc

      return ""

    }
  }

  ##
  ## botonchan
  ##   added: 1.3.24
  ##
  ## Args:    channel
  ## Returns: 1 if the bot is on the given channel
  ##          0 otherwise
  ##
  proc compat:botonchan {args} {
    global botnick

    BadArgs $args 1 1 "botonchan channel"
    set chan [lindex $args 0]
    if {![validchan $chan]} then {
      error "illegal channel: $chan"
    } elseif {[onchan $botnick $chan]} then {
      return 1
    }
    return 0
  }

  ##
  ## putquick
  ##   added: 1.3.24
  ##
  ## Args:    text
  ## Returns: nothing
  ##
  proc compat:putquick {args} {
    BadArgs $args 1 1 "putquick text"
    putserv [lindex $args 0]
    return
  }

  ##
  ## botisvoice
  ##   added: 1.3.25
  ##
  ## Args:    channel
  ## Returns: 1 if the bot is voice (+v) on the given channel
  ##          0 otherwise
  ##
  proc compat:botisvoice {args} {
    global botnick

    BadArgs $args 1 1 "botisvoice channel"
    set chan [lindex $args 0]
    if {![validchan $chan]} then {
      error "illegal channel: $chan"
    } elseif {[isvoice $botnick $chan]} then {
      return 1
    }
    return 0
  }

  ##
  ## islinked
  ##   added: 1.3.25
  ##
  ## Args:    bot
  ## Returns: 1 if the given bot is linked to the botnet
  ##          0 otherwise
  ##
  proc compat:islinked {args} {
    BadArgs $args 1 1 "islinked bot"
    if {[lsearch -exact [string tolower [bots]] \
         [string tolower [lindex $args 0]]] != -1} then {
      return 1
    }
    return 0
  }

  ##
  ## isbotnick
  ##   added: 1.3.28
  ##
  ## Args:    nick
  ## Returns: 1 if the given nick matches the bot's current nickname
  ##          0 otherwise
  ##
  proc compat:isbotnick {args} {
    global botnick

    BadArgs $args 1 1 "isbotnick nick"
    if {![string compare [string tolower $botnick] \
          [string tolower [lindex $args 0]]]} then {
      return 1
    }
    return 0
  }

  ##
  ## duration
  ##   added: 1.3.25, buggy up to 1.4.0
  ##
  ## Args:    seconds
  ## Returns: the number of seconds in years, weeks, days, hours, minutes,
  ##          and seconds
  ##
  proc compat:duration {args} {
    BadArgs $args 1 1 "duration seconds"
    set result ""
    set sec [lindex $args 0]
    if {$sec >= 31536000} then {
      set tmp [expr $sec / 31536000]
      if {$tmp > 1} then {
        append result "$tmp years "
      } else {
        append result "$tmp year "
      }
      set sec [expr $sec - ($tmp * 31536000)]
    }
    if {$sec >= 604800} then {
      set tmp [expr $sec / 604800]
      if {$tmp > 1} then {
        append result "$tmp weeks "
      } else {
        append result "$tmp week "
      }
      set sec [expr $sec - ($tmp * 604800)]
    }
    if {$sec >= 86400} then {
      set tmp [expr $sec / 86400]
      if {$tmp > 1} then {
        append result "$tmp days "
      } else {
        append result "$tmp day "
      }
      set sec [expr $sec - ($tmp * 86400)]
    }
    if {$sec >= 3600} then {
      set tmp [expr $sec / 3600]
      if {$tmp > 1} then {
        append result "$tmp hours "
      } else {
        append result "$tmp hour "
      }
      set sec [expr $sec - ($tmp * 3600)]
    }
    if {$sec >= 60} then {
      set tmp [expr $sec / 60]
      if {$tmp > 1} then {
        append result "$tmp minutes "
      } else {
        append result "$tmp minute "
      }
      set sec [expr $sec - ($tmp * 60)]
    }
    if {$sec} then {
      if {$sec > 1} then {
        append result "$sec seconds"
      } else {
        append result "$sec second"
      }
    }
    return [string trimright $result]
  }

  ##
  ## putkick
  ##   added 1.4.0  [Note: this isn't 100% emulation]
  ##
  ## Args:    channel nick?s? ?comment?
  ## Returns: nothing
  ##
  proc compat:putkick {args} {
    BadArgs $args 3 4 "putkick channel nick?s? ?comment?"

    set chan [lindex $args 0]
    if {![validchan $chan]} then {
      error "illegal channel: $chan"
    } elseif {![botisop $chan]} then {
      error "need op"
    }
    set comment [lidnex $args 2]
    if {![string compare "" $comment]} then {
      set comment ""
    }
    foreach nick [split [lindex $args 1] ,] {
      if {[onchan $nick $chan]} then {
        putquick "KICK $chan $nick $comment"
      }
    }
    return
  }

  ##
  ## Check argument usage for commands
  ##
  ## Args:    argument list, min args, max args, command usage example
  ## Returns: nothing
  ##
  proc BadArgs {arg min max example} {
    if {([set length [llength $arg]] < $min) || ($length > $max)} then {
      error "wrong # args: should be \"$example\""
    }
    return
  }

  # version "command list"
  array set CommandList {
    1031500 "erasenotes listnotes"
    1032400 "botonchan putquick"
    1032500 "botisvoice islinked"
    1032800 "isbotnick"
    1040000 "duration putkick"
  }

  ##
  ## Init module specific things at load time
  ##
  ## Args:    action {load|unload}
  ## Returns: nothing
  ##
  proc InitTable {action} {
    global argv0 numversion
    variable NamespaceCurrent
    variable CommandList

    switch -exact -- $action {
      load {
        if {![info exists NamespaceCurrent]} then {
          set NamespaceCurrent [namespace current]
        }
        if {![info exists argv0]} then {
          # Work around for buggy tcl variables in certain bots
          catch {set numversion}
          foreach {name data} [array get CommandList] {
            foreach command $data {
              if {[string compare "" [info commands compat:$command]]} then {
                if {$numversion < $name} then {
                  lappend GlobalRenameList $command
                } else {
                  rename compat:$command ""
                }
              }
            }
          }
          if {[info exists GlobalRenameList]} then {
            set Eval "namespace eval :: \{rename ${NamespaceCurrent}::BadArgs BadArgs\}"
            eval $Eval
            foreach command $GlobalRenameList {
              set Eval "namespace eval :: \{rename ${NamespaceCurrent}::compat:$command $command\}"
              eval $Eval
            }
          } elseif {[string compare "" [info commands BadArgs]]} then {
            rename BadArgs ""
          }
        }
      }
      unload {
        error "This module cannot be unloaded."
      }
    }
    return
  }

} ;# namespace compat

return
