###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2001  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: wakeup.tcl,v 1.3 2001/06/07 08:19:26 tothwolf Exp $
##

# name: wakeup
# version: 1.4.0.0
# config: wakeup.conf
# description: Anti-idle for bots that uses a /msg of `wakeup' followed by \
#	a random string of 30-40 characters.
# requires: isbotnick putmsg randstring timerExists timerId

namespace eval wakeup {

  ##
  ## Ignore wakeup message
  ##
  ## Args:    nick, uhost, handle, arg
  ## Returns: 0
  ##
  proc msg:wakeupkey {nick uhost hand arg} {
    return 0
  }

  ##
  ## Send a message to a bot
  ##
  ## Args:    none
  ## Returns: nothing
  ##
  proc AntiIdle {} {
    global botnick
    variable configData

    # Random text to send out
    set text [randstring [expr [rand 30] + 11] "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789`~!@#$%^&*()-_=+\\|\[\{\]\}:;'\",<.>/? \t"]

    # Search all channels for a bot, send a message if we find one
    foreach chan [channels] {
      if {[llength [set bots [chanlist $chan b]]]} then {
        foreach bot [chanlist $chan b] {
          if {![isbotnick $bot]} then {
            putmsg $bot "$configData(wakeupkey) $text"
            # Start another loop
            StartAntiIdle
            return
          }
        }
      }
    }

    # Send a message to ourself as a last resort
    putmsg $botnick "$configData(wakeupkey) $text"

    # Start another loop
    StartAntiIdle
    return
  }

  ##
  ## Start an anti-idle loop
  ##
  ## Args:    none
  ## Returns: nothing
  ##
  proc StartAntiIdle {} {
    variable NamespaceCurrent
    variable configData

    if {![timerExists ${NamespaceCurrent}::AntiIdle utimers]} then {
      if {$configData(offset)} then {
        utimer [expr $configData(delay) + [rand $configData(offset)]] ${NamespaceCurrent}::AntiIdle
      } else {
        utimer $configData(delay) ${NamespaceCurrent}::AntiIdle
      }
    }
    return
  }

  ##
  ## Trace configuration changes
  ##
  ## Args:    name1, name2, operation
  ## Returns: nothing
  ##
  proc TraceConfigData {name1 name2 op} {
    variable configData
    variable oldConfigData

    if {[string compare $oldConfigData($name2) $configData($name2)]} then {
      switch -exact -- $name2 {
        wakeupkey {
          BindTable unbind msg:key
          set oldConfigData($name2) $configData($name2)
          BindTable bind msg:key
        }
      }
    }
    return
  }

  ##
  ## Trace and respond to variable changes
  ##
  ## Args:    action {variable|vdelete}
  ## Returns: nothing
  ##
  proc TraceTable {action} {
    variable NamespaceCurrent
    variable configData

    trace $action configData w ${NamespaceCurrent}::TraceConfigData
    return
  }

  ##
  ## Init module specific things at load time
  ##
  ## Args:    action {load|unload}
  ## Returns: nothing
  ##
  proc InitTable {action} {
    variable NamespaceCurrent
    variable configData
    variable oldConfigData

    switch -exact -- $action {
      load {
        if {![info exists NamespaceCurrent]} then {
          set NamespaceCurrent [namespace current]
        }
        foreach {name data} [array get configData] {
          if {![info exists oldConfigData($name)]} then {
            set oldConfigData($name) $data
          }
        }
        StartAntiIdle
      }
    }
    return
  }

  ##
  ## Bind procs to commands and events
  ##
  ## Args:    action {bind|unbind}, what {type|*}, args {...}
  ## Returns: nothing
  ##
  proc BindTable {action what args} {
    variable NamespaceCurrent
    variable oldConfigData

    if {![string compare * $what]} then {
      set what msg:key
    }
    foreach type $what {
      switch -exact -- $type {
        msg:key {
          $action msg -|- $oldConfigData(wakeupkey) ${NamespaceCurrent}::msg:wakeupkey
        }
      }
    }
    return
  }

  # Configuration data defaults
  array set configDataDefaults {
    delay 200
    offset 300
    wakeupkey wakeup
  }

  # Configuration data descriptions
  array set configDataDesc {
    delay "\[seconds\] time added to random offset"
    offset "\[seconds\] number used to generate random offset (0 to disable)"
    wakeupkey "\[word\] word to bind and send with random text"
  }

} ;# namespace wakeup

return
