###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2001  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: hunt.tcl,v 1.2 2001/06/02 03:15:07 tothwolf Exp $
##

# name: wrpg-hunt
# version: 1.0.0.0
# config:
# description:
# requires: this_module_needs_to_be_rewritten

namespace eval wrpg-hunt {

  ##
  ## Handle food data
  ##
  ## Args:    action {add|del|set|list|test|realname}, args
  ## Returns: data passed from sub procs
  ##
  proc foodData {action args} {
    switch -exact -- $action {
      add {
        return [foodDataAdd $args]
      }
      del {
        return [foodDataDel $args]
      }
      set {
        return [foodDataSet $args]
      }
      get {
        return [foodDataGet $args]
      }
      list {
        return [foodDataList $args]
      }
      test {
        return [foodDataTest $args]
      }
      realname {
        return [foodDataRealname $args]
      }
      format {
        return foodDataformat $args]
      }
    }
    return
  }

  ##
  ## Add food data to the food list
  ##
  ## Args:    args {name, amount}
  ## Returns: {1 realname} if name added to food list
  ##          0 if name is not in hunt list
  ##          {-1 realname} if amount is invalid
  ##
  proc foodDataAdd {args} {
    variable dataFormats
    variable fooddata

    set realname [huntData realname [lindex $args 0]]
    if {[string compare "" $realname]} then {
      set amount [lindex $args 1]
      if {![string compare "" $amount]} then {
        set amount [huntData value $realname food]
      } elseif {(![isNumber $amount]) || ($amount < 1)} then {
        return [list -1 $realname]
      }

      if {[info exists fooddata($realname)]} then {
#        set oldamount [dataFormatValue $dataFormats(fooddata) $fooddata($realname) amount]
# FIXME
#        set fooddata($realname) [dataFormatReplace $dataFormats(fooddata) $fooddata($realname) amount [expt $oldamount + $amount]]
        return [list 1 $realname]
      } else {
        set timestamp [clock seconds]
# FIXME
#        set fooddata($realname) [dataFormatReplace $dataFormats(fooddata) $fooddata($realname) "amount timestamp" "$amount [clock seconds]"]
        return [list 1 $realname]
      }
    }
    return 0
  }

  ##
  ## Remove food data from the food list
  ##
  ## Args:    name, args {amount}
  ## Returns: {2 realname} if name removed from food list
  ##          {1 realname} if amount removed from food list
  ##          0 if name is not in hunt list
  ##          {-1 realname} if amount is invalid
  ##          {-2 realname} if name is not in food list
  ##
  proc foodDataDel {name args} {
    variable fooddata

    set realname [huntData name $name]
    if {[string compare "" $realname]} then {
      if {[info exists fooddata($realname)]} then {
        set amount [lindex $args 0]
        if {(![string compare "" $amount]) || \
            (![string compare $amount [lindex $fooddata($name) 0]])} then {
          unset fooddata($realname)
          return [list 2 $realname]
        } elseif {([isNumber $amount]) && \
                  ($amount >= 1) && \
                  ($amount <= [lindex $fooddata($realname) 0])} then {
          set fooddata($realname) [lreplace $fooddata($realname) 0 0 [expr [lindex $fooddata($realname) 0] - $amount]]
          return [list 1 $realname]
        } else {
          return [list -1 $realname]
        }
      } else {
        return [list -2 $realname]
      }
    }
    return 0
  }

  ##
  ## Set food data amount in the food list
  ##
  ## Args:    name, args {amount}
  ## Returns: {1 realname} if name set to amount
  ##          0 if name is not in hunt list
  ##          {-1 realname} if amount is invalid
  ##          {-2 realname} if name is not in food list
  ##
  proc foodDataSet {name args} {
    variable fooddata

    set realname [huntData name $name]
    if {[string compare "" $realname]} then {
      if {[info exists fooddata($realname)]} then {
        set amount [lindex $args 0]
        if {([isNumber $amount]) && ($amount >= 0)} then {
          set fooddata($realname) [lreplace $fooddata($realname) 0 0 $amount]
          return [list 1 $realname]
        } else {
          return [list -1 $realname]
        }
      } else {
        return [list -2 $realname]
      }
    }
    return 0
  }

  ##
  ## Remove food data from the food list
  ##
  ## Args:    name, args {amount}
  ## Returns: {3 data} if what is a valid name in food list
  ##          {2 count list} if what is a valid day count in food list
  ##          {1 count list} if what is not specified
  ##          0 if name is not in hunt list
  ##          -1 if food list is empty
  ##          -2 if no food data has the specified day count
  ##          -3 if food never spoils
  ##          {-4 realname} if name is not in food list
  ##
  proc foodDataList {name args} {
    variable fooddata

    set count 0
    if {![string compare "" $what]} then {
      foreach name [lsort [array names fooddata]] {
        foreach {amount time} $fooddata($name) {break}
        if {$configData(foodspoil)} then {
          lappend list "$name \[$amount:[daysLeft $time $configData(foodspoil)]\],"
        } else {
          lappend list "$name \[$amount\],"
        }
        incr count
      }
      if {$count} then {
        return [list 1 $count $list]
      } else {
        return -1
      }
    } elseif {[isNumber $what]} then {
      if {$configData(foodspoil)} then {
        foreach name [lsort [array names fooddata]] {
          foreach {amount time} $fooddata($name) {break}
          set age [daysLeft $time $configData(foodspoil)]
          if {![string compare $what $age]} then {
            lappend list "$name \[$amount:$age\],"
            incr count
          }
        }
        if {$count} then {
          return [list 2 $count $list]
        } else {
          return -2
        }
      } else {
        return -3
      }
    } else {
      set realname [huntData name $name]
      if {[string compare "" $realname]} then {
        if {[info exists fooddata($realname)]} then {
          foreach {amount time} $fooddata($realname) {break}
          if {$configData(foodspoil)} then {
            return [list 3 "$realname \[$amount:[daysLeft $time $configData(foodspoil)]\]"]
          } else {
            return [list 3 "$name \[$amount\]"]
          }
        } else {
          return [list -4 $realname]
        }
      }
    }
    return 0
  }

  ##
  ## Test if food data exists in the food list
  ##
  ## Args:    name, args
  ## Returns: 1 if name is in the food list
  ##          0 if name is not in the hunt list
  ##          -1 if name is not in the food list
  ##
  proc foodDataTest {name args} {
    variable fooddata

    set realname [huntData name $name]
    if {[string compare "" $realname]} then {
      if {[info exists fooddata($realname)]} then {
        return 1
      } else {
        return -1
      }
    }
    return 0
  }

  ##
  ## Find realname for food data in the food list
  ##
  ## Args:    name, args
  ## Returns: realname if name is in the food list
  ##          "" if name is not in the food list
  ##
  proc foodDataRealname {name args} {
    variable NamespaceCurrent

    set realname [arrayFindElementName ${NamespaceCurrent}::fooddata $name]
    if {[string compare "" $realname]} then {
      return $realname
    }
    return
  }







#################################################################




  ##
  ## Return a random animal for the given group
  ##
  ## Args:    args {group}
  ## Returns: random animal
  ##
  proc randomAnimal {args} {
    variable dataFormats
    variable huntdata

    set count 0
    set group [lindex $args 0]
    foreach {name data} [array get huntdata] {
      if {([string compare "" $name]) && \
          ([dataFormatValue $dataFormats(huntdata) $data hunt] == 1) && \
          (([string compare "" $group]) || \
           (![string compare $group \
              [dataFormatValue $dataFormats(huntdata) $data group]]))} then {
        lappend list $i
        incr count
      }
    }
    if {$count} then {
      return [lindex [lsort $list] [rand $count]]
    }
    return
  }

  # Configuration data defaults
  array set configDataDefaults {
    foodspoil 10
  }

  # Configuration data descriptions
  array set configDataDesc {
    foodspoil "\[days\] how many days should pass before food spoils"
  }

  # Set data format information
  array set dataFormats {
    fooddata "{amount 1} {timestamp 0}"
    huntdata "{group 1} {hits 1} {food 1} {hunt 1} {sound 0} {}"
  }

  # Set data file information
  array set dataOptions {
    fooddata "array wrpg-food.data food"
    huntdata "array wrpg-hunt.data hunt"
  }


###############################################################################


  ##
  ## Handle food data
  ##
  ## Args:    action {add|del|set|test|list}, name, args
  ## Returns: data passed from sub procs
  ##
  proc foodData {action name args} {
    switch -exact -- $action {
      add {
        return [addFoodData $name $args]
      }
      del {
        return [delFoodData $name $args]
      }
      set {
        return [setFoodData $name $args]
      }
      test {
        return [testFoodData $name $args]
      }
      list {
        return [listFoodData $name $args]
      }
    }
    return
  }

  ##
  ## Add food data to the food list
  ##
  ## Args:    name, args {amount}
  ## Returns: {1 realname} if name added to food list
  ##          0 if name is not in hunt list
  ##          {-1 realname} if amount is invalid
  ##
  proc addFoodData {name args} {
    variable huntdata
    variable fooddata

    set realname [arrayFindElementName huntdata $name]
    if {[string compare "" $realname]} then {
      set amount [lindex $args 0]
      if {![string compare "" $amount]} then {
        set amount [lindex $huntdata($realname) 2]
      } elseif {(![isNumber $amount]) || ($amount < 1)} then {
        return [list -1 $realname]
      }
      if {[info exists fooddata($realname)]} then {
        set fooddata($realname) [lreplace $fooddata($realname) 0 0 [expr [lindex $fooddata($realname) 0] + $amount]]
        return [list 1 $realname]
      } else {
        set fooddata($realname) "$amount [clock seconds]"
        return [list 1 $realname]
      }
    }
    return 0
  }

  ##
  ## Remove food data from the food list
  ##
  ## Args:    name, args {amount}
  ## Returns: {2 realname} if name removed from food list
  ##          {1 realname} if amount removed from food list
  ##          0 if name is not in hunt list
  ##          {-1 realname} if amount is invalid
  ##          {-2 realname} if name is not in food list
  ##
  proc delFoodData {name args} {
    variable fooddata

    set realname [arrayFindElementName huntdata $name]
    if {[string compare "" $realname]} then {
      if {[info exists fooddata($realname)]} then {
        set amount [lindex $args 0]
        if {(![string compare "" $amount]) || \
            (![string compare $amount [lindex $fooddata($name) 0]])} then {
          unset fooddata($realname)
          return [list 2 $realname]
        } elseif {([isNumber $amount]) && \
                  ($amount >= 1) && \
                  ($amount <= [lindex $fooddata($realname) 0])} then {
          set fooddata($realname) [lreplace $fooddata($realname) 0 0 [expr [lindex $fooddata($realname) 0] - $amount]]
          return [list 1 $realname]
        } else {
          return [list -1 $realname]
        }
      } else {
        return [list -2 $realname]
      }
    }
    return 0
  }

  ##
  ## Set food data amount in the food list
  ##
  ## Args:    name, args {amount}
  ## Returns: {1 realname} if name set to amount
  ##          0 if name is not in hunt list
  ##          {-1 realname} if amount is invalid
  ##          {-2 realname} if name is not in food list
  ##
  proc setFoodData {name args} {
    variable fooddata

    set realname [arrayFindElementName huntdata $name]
    if {[string compare "" $realname]} then {
      if {[info exists fooddata($realname)]} then {
        set amount [lindex $args 0]
        if {([isNumber $amount]) && ($amount >= 0)} then {
          set fooddata($realname) [lreplace $fooddata($realname) 0 0 $amount]
          return [list 1 $realname]
        } else {
          return [list -1 $realname]
        }
      } else {
        return [list -2 $realname]
      }
    }
    return 0
  }

  ##
  ## Test if food data exists in the food list
  ##
  ## Args:    name, args
  ## Returns: 1 if name is in the food list
  ##          0 if name is not in the hunt list
  ##          -1 if name is not in the food list
  ##
  proc testFoodData {name args} {
    variable fooddata

    set realname [arrayFindElementName huntdata $name]
    if {[string compare "" $realname]} then {
      if {[info exists fooddata($realname)]} then {
        return 1
      } else {
        return -1
      }
    }
    return 0
  }

  ##
  ## Remove food data from the food list
  ##
  ## Args:    name, args {amount}
  ## Returns: {3 data} if what is a valid name in food list
  ##          {2 count list} if what is a valid day count in food list
  ##          {1 count list} if what is not specified
  ##          0 if name is not in hunt list
  ##          -1 if food list is empty
  ##          -2 if no food data has the specified day count
  ##          -3 if food never spoils
  ##          {-4 realname} if name is not in food list
  ##
  proc listFoodData {name args} {
    variable fooddata

    set count 0
    if {![string compare "" $what]} then {
      foreach name [lsort [array names fooddata]] {
        foreach {amount time} $fooddata($name) {break}
        if {$configData(foodspoil)} then {
          lappend list "$name \[$amount:[daysLeft $time $configData(foodspoil)]\],"
        } else {
          lappend list "$name \[$amount\],"
        }
        incr count
      }
      if {$count} then {
        return [list 1 $count $list]
      } else {
        return -1
      }
    } elseif {[isNumber $what]} then {
      if {$configData(foodspoil)} then {
        foreach name [lsort [array names fooddata]] {
          foreach {amount time} $fooddata($name) {break}
          set age [daysLeft $time $configData(foodspoil)]
          if {![string compare $what $age]} then {
            lappend list "$name \[$amount:$age\],"
            incr count
          }
        }
        if {$count} then {
          return [list 2 $count $list]
        } else {
          return -2
        }
      } else {
        return -3
      }
    } else {
      set realname [arrayFindElementName huntdata $name]
      if {[string compare "" $realname]} then {
        if {[info exists fooddata($realname)]} then {
          foreach {amount time} $fooddata($realname) {break}
          if {$configData(foodspoil)} then {
            return [list 3 "$realname \[$amount:[daysLeft $time $configData(foodspoil)]\]"]
          } else {
            return [list 3 "$name \[$amount\]"]
          }
        } else {
          return [list -4 $realname]
        }
      }
    }
    return 0
  }

  ##
  ## Return a random animal for the given group
  ##
  ## Args:    args {group}
  ## Returns: random animal
  ##
  proc randomAnimal {args} {
    variable huntdata

    set count 0
    set group [lindex $args 0]
    foreach {name data} [array get huntdata] {
      if {([string compare "" $name]) && ([lindex $data 3]) && \
          (([string compare "" $group]) || \
           (![string compare $group [lindex $data 0]]))} then {
        lappend list $i
        incr count
      }
    }
    if {$count} then {
      return [lindex [lsort $list] [rand $count]]
    }
    return
  }

  # Configuration data defaults
  array set configDataDefaults {
    foodspoil 10
  }

  # Configuration data descriptions
  array set configDataDesc {
    foodspoil "\[days\] how many days should pass before food spoils"
  }

  # Set datafile information
  array set dataOptions {
    huntdata "array wrpg-hunt.data hunt"
    fooddata "array wrpg-food.data food"
  }



###############################################################################

  # sniff out prey; pub
  proc pub:sniff {nick uhost hand chan arg} {
#    variable huntdata
#    variable huntinfo
#    variable NamespaceCurrent
#
#    if {[validHandle $hand} then {
#      if {[hp test1 $hand 0] != 2} then {
#        if {!$huntinfo(active)} then {
#          set arg [split [string trimright $arg]]
#          if {![string length $arg]} then {
#            set animal [random_animal]
#            if {[string length $animal]} then {
#              set huntinfo(animal) $animal
#              set huntinfo(time) [clock seconds]
#              putmsg $chan "$nick catches the scent of [grammar_check2 0 [join $animal]] (It has [grammar_check1 0 [lindex $huntdata($animal) 1] hit])"
#            } else {
#              putnot $nick "No animals are in the hunt list."
#            }
#          } elseif {[isNumber $arg]} then {
#            set animal [random_animal $arg]
#            if {[string length $animal]} then {
#              set huntinfo(animal) $animal
#              set huntinfo(time) [clock seconds]
#              putmsg $chan "$nick catches the scent of [grammar_check2 0 [join $animal]] (It has [grammar_check1 0 [lindex $huntdata($animal) 1] hit])"
#            } else {
#              putnot $nick "No animals are in the hunt list for group $arg"
#            }
#          } else {
#            set animal [arrayFindElementName ${NamespaceCurrent}::huntdata $arg]
#            if {[string length $animal]} then {
#              set huntinfo(animal) $animal
#              set huntinfo(time) [clock seconds]
#              putmsg $chan "$nick catches the scent of [grammar_check2 0 [join $animal]] (It has [grammar_check1 0 [lindex $huntdata($animal) 1] hit])"
#            } else {
#              putnot $nick "I don't know what [grammar_check2 0 $arg] is."
#            }
#          }
#        } else {
#          putnot $nick "A hunt for [grammar_check2 0 [join $huntinfo(animal)]] is already in progress."
#        }
#      } else {
#        putmsg $chan "Sorry $nick, you are too weak to hunt."
#      }
#      return 1
#    }
#    return 0
  }

  # start a hunt
  proc pub:hunt {nick uhost hand chan arg} {
#    variable huntdata
#    variable huntinfo
#    variable NamespaceCurrent
#
#    if {[validHandle $hand]} then {
#      if {[hp test1 $hand 0] != 2} then {
#        if {!$huntinfo(active)} then {
#          set arg [split [string trimright $arg]]
#          if {[string length $arg]} then {
#            set animal [arrayFindElementName ${NamespaceCurrent}::huntdata $arg]
#          } elseif {[string length $huntinfo(animal)]} then {
#            set animal $huntinfo(animal)
#          }
#          if {[info exists animal]} then {
#            if {[string length $animal]} then {
#              if {[lev list $hand ""] >= [lindex $huntdata($animal) 0]} then {
#                config_dice use ${NamespaceCurrent}::hunt_dice
#                set diceopt [lindex $huntdata($animal) 5]
#                if {[string length $diceopt]} then {
#                  config_dice configure $diceopt
#                  set huntinfo(diceopt) 1
#                }
#                set huntinfo(active) 1
#                set huntinfo(caught) 0
#                set huntinfo(channel) $chan
#                set huntinfo(animal) $animal
#                set huntinfo(time) [clock seconds]
#                set huntinfo(hits_total) [lindex $huntdata($animal) 1]
#                set huntinfo(hits_remaining) $huntinfo(hits_total)
#                set huntinfo(hits_scored) 0
#                putmsg $chan "Hunt for [grammar_check2 0 [join $animal]] now active! (It has [grammar_check1 0 $huntinfo(hits_total) hit])"
#              } else {
#                putnot $nick "Your level is not high enough to start a hunt for [grammar_check2 0 [join $animal]]"
#              }
#            } else {
#              putnot $nick "I don't know what [grammar_check2 0 $arg] is."
#            }
#          } else {
#            putnot $nick "You must sniff out an animal or specify what to hunt."
#          }
#        } else {
#          putnot $nick "A hunt for [grammar_check2 0 [join $huntinfo(animal)]] is already in progress."
#        }
#      } else {
#        putmsg $chan "Sorry $nick, you are too weak to hunt."
#      }
#      return 1
#    }
#    return 0
  }

  # immediately terminate a hunt
  proc pub:endhunt {nick uhost hand chan arg} {
# FIXME: $huntinfo(channel)
#    variable huntdata
#    variable huntinfo
#    variable NamespaceCurrent
#
#    if {[validHandle $hand]} then {
#      if {[hp test1 $hand 0] != 2} then {
#        if {$huntinfo(active)} then {
#          set arg [split [string trimright $arg]]
#          if {[string length $arg]} then {
#            set animal [arrayFindElementName ${NamespaceCurrent}::huntdata $arg]
#          } else {
#            set animal $huntinfo(animal)
#          }
#          if {[string length $animal]} then {
#            if {[lev list $hand ""] >= [lindex $huntdata($animal) 0]} then {
#              if {![string compare $animal $huntinfo(animal)]} then {
#                putmsg $chan "Hunt for [grammar_check2 0 [join $huntinfo(animal)]] ended, [grammar_check3 [grammar_check1 0 $huntinfo(hits_scored) hit]] scored out of $huntinfo(hits_total)."
#                clear_hunt
#              } else {
#                putnot $nick "[grammar_check2 1 [join $animal]] is not being hunted, [grammar_check2 0 [join $huntinfo(animal)]] is."
#              }
#            } else {
#              putnot $nick "Your level is not high enough to end a hunt for [grammar_check2 0 [join $animal]]"
#            }
#          } else {
#            putnot $nick "I don't know what [grammar_check2 0 $arg] is."
#          }
#        } else {
#          putnot $nick "There wasn't a hunt active."
#        }
#      } else {
#        putnot $chan "Sorry $nick, you are too weak to hunt."
#      }
#      return 1
#    }
#    return 0
  }

  # register caught prey from the hunt; pub
  proc pub:caught {nick uhost hand chan arg} {
#    variable conf
#    variable huntinfo
#    variable huntdata
#    variable fooddata
#    variable NamespaceCurrent
#
#    if {[validHandle $hand]} then {
#      if {[hp test1 $hand 0] != 2} then {
#        set arg [split [string trimright $arg]]
#        if {[string length $arg]} then {
#          set animal [arrayFindElementName ${NamespaceCurrent}::huntdata $arg]
#        } elseif {$huntinfo(active)} then {
#          set animal $huntinfo(animal)
#        }
#        if {[info exists animal]} then {
#          if {[string length $animal]} then {
#            if {$huntinfo(active)} then {
#              if {!$conf(autocaught)} then {
#                if {![string compare $animal $huntinfo(animal)]} then {
#                  if {[hunt_catch $animal]} then {
#                    putmsg $chan "There is now enough [join $animal] for [lindex $fooddata($animal) 0] wolves."
#                  } else {
#                    putnot $nick "The [join $animal] hasn't been caught yet!"
#                  }
#                } else {
#                  putnot $nick "[grammar_check2 1 [join $animal]] was not caught, [grammar_check2 0 [join $huntinfo(animal)]] was."
#                }
#              } else {
#                putnot $nick "The caught command is unnecessary since autocaught is enabled."
#              }
#            } elseif {[lindex $huntdata($animal) 3]} then {
#              if {[hunt_catch $animal]} then {
#                putmsg $chan "There is now enough [join $animal] for [lindex $fooddata($animal) 0] wolves."
#              }
#            } else {
#              putnot $nick "There isn't a hunt active."
#            }
#          } else {
#            putnot $nick "I don't know what [grammar_check2 0 [join $animal]] is."
#          }
#        } else {
#          putnot $nick "You must hunt an animal or specify what to catch."
#        }
#      } else {
#        putmsg $chan "Sorry $nick, you are too weak to hunt."
#      }
#      return 1
#    }
#    return 0
  }

  # register caught prey from the hunt
  proc hunt_catch {animal} {
#    variable huntinfo
#    variable huntdata
#    variable fooddata
#    variable fooddata_changed
#
#    if {($huntinfo(caught)) || (![lindex $huntdata($animal) 3])} then {
#      if {![info exists fooddata($animal)]} then {
#        set fooddata($animal) "[lindex $huntdata($animal) 2] [clock seconds]"
#      } else {
#        set fooddata($animal) "[expr [lindex $fooddata($animal) 0] + [lindex $huntdata($animal) 2]] [clock seconds]"
#      }
#      set fooddata_changed 1
#      if {$huntinfo(active)} then {
#        clear_hunt
#      }
#      return 1
#    }
#    return 0
  }


  # eat prey to increase user's fp
  proc pub:eat {nick uhost hand chan arg} {
#    variable conf
#    variable fooddata
#    variable fooddata_changed
#    variable NamespaceCurrent
#
#    if {[validHandle $hand]} then {
#      set arg [split [string trimright $arg]]
#      if {![string length $arg]} then {
#        if {$conf(foodspoil)} then {
#          foreach i [lsort [array names fooddata]] {
#            lappend list "[join $i] \[[lindex $fooddata($i) 0]:[days_left [lindex $fooddata($i) 1] $conf(foodspoil)]\],"
#          }
#        } else {
#          foreach i [lsort [array names fooddata]] {
#            lappend list "[join $i] \[[lindex $fooddata($i) 0]\],"
#          }
#        }
#        if {[info exists list]} then {
#          formatLists putmsg $chan [string trimright [join $list] ,] 160 , , " " "The food available is: " "(cont): "
#        } else {
#          putmsg $chan "The food list is empty :("
#        }
#      } else {
#        if {[isNumber [lindex $arg end]]} then {
#          set animal [arrayFindElementName ${NamespaceCurrent}::huntdata [lrange $arg 0 [expr [llength $arg] - 2]]]
#          set amount [lindex $arg end]
#        } else {
#          set animal [arrayFindElementName ${NamespaceCurrent}::huntdata $arg]
#          set amount 1
#        }
#        #FIXME: $animal == ""
#        if {[info exists fooddata($animal)]} then {
#          set count [lindex $fooddata($animal) 0]
#          if {$count > 0} then {
#            if {([days_left [lindex $fooddata($animal) 1] $conf(foodspoil)] > 0) || (!$conf(foodspoil))} then {
#              if {![matchattr $hand F]} then {
#                if {[fp test2 $hand 0] != 2} then {
#                  if {[expr ([fp limit $hand 0] - [fp list $hand 0]) >= $amount]} then {
#                    if {[isNumber $amount]} then {
#                      if {[lindex $fooddata($animal) 0] >= $amount} then {
#                        #FIXME: need to do this all at once
#                        for {set count2 $amount} {$count2 > 0} {incr count2 -1} {
#                          if {$count > 1} then {
#                            if {[fp incr $hand 1]} then {
#                              set fooddata($animal) [lreplace $fooddata($animal) 0 0 [expr [lindex $fooddata($animal) 0] - 1]]
#                              set fooddata_changed 1
#                              incr count -1
#                            }
#                          } else {
#                            if {[fp incr $hand 1]} then {
#                              set fooddata($animal) [lreplace $fooddata($animal) 0 0 0]
#                              set fooddata_changed 1
#                              incr count -1
#                            }
#                          }
#                        }
#                        if {$count < 1} then {
#                          if {$conf(foodbury)} then {
#                            unset fooddata($animal)
#                          }
#                          putmsg $chan "$nick eats the last of the [join $animal]."
#                        } elseif {$count == 1} then {
#                          putmsg $chan "$nick eats some of the [join $animal], there is only enough left for one more wolf."
#                        } elseif {$count > 1} then {
#                          putmsg $chan "$nick eats some of the [join $animal], there is enough left for [lindex $fooddata($animal) 0] more wolves."
#                        }
#                      } else {
#                        putnot $nick "There is not that much [join $animal] to eat."
#                      }
#                    } else {
#                      putnot $nick "Invalid count of [join $animal] to eat."
#                    }
#                  } else {
#                    putmsg $chan "Sorry $nick, but you are too full to eat that much [join $animal]."
#                  }
#                } else {
#                  putmsg $chan "Sorry $nick, but you are too full to eat."
#                }
#              } else {
#                putnot $nick "You don't need to eat anything."
#              }
#            } else {
#              putmsg $chan "The [join $animal] has spoiled, please bury it."
#            }
#          } else {
#            putmsg $chan "There is nothing but a carcass left of the [join $animal], please bury it."
#          }
#        } else {
#          putmsg $chan "There is no [join $animal] to eat."
#        }
#      }
#      return 1
#    }
#    return 0
  }

  # bury prey that has been eaten or has spoiled
  proc pub:bury {nick uhost hand chan arg} {
#    variable conf
#    variable fooddata
#    variable fooddata_changed
#    variable NamespaceCurrent
#
#    if {[validHandle $hand]} then {
#      set arg [split [string trimright $arg]]
#      if {[string length $arg]} then {
#        set animal [arrayFindElementName ${NamespaceCurrent}::huntdata $arg]
#        if {[string length $animal]} then {
#          if {[info exists fooddata($animal)]} then {
#            if {([lindex $fooddata($animal) 0] <= 0) || \
#                (!$conf(foodspoil)) || \
#                ([days_left [lindex $fooddata($animal) 1] $conf(foodspoil)] <= 0)} then {
#              putmsg $chan "$nick buries the [join $animal]"
#              unset fooddata($animal)
#              set fooddata_changed 1
#            } else {
#              putnot $nick "There is still [join $animal] left."
#            }
#          } else {
#            putnot $nick "Theres no [join $animal] to bury."
#          }
#        } else {
#          putnot $nick "I don't know what [grammar_check2 0 $arg] is."
#        }
#      } else {
#        putnot $nick "You must specify the animal to bury."
#      }
#      return 1
#    }
#    return 0
  }

  # score a hunt
  proc hunt_dice {nick uhost hand chan result} {
#    variable conf
#    variable huntinfo
#    variable huntdata
#    variable NamespaceParent
#
#    if {$huntinfo(active)} then {
#      if {!$huntinfo(caught)} then {
#        if {[hp test1 $hand 0] != 2} then {
#          if {(($conf(allownonpack)) && (![validuser $hand])) || ([lev list $hand 0])} then {
#            set huntinfo(time) [clock seconds]
#            set hunt_hits1 0
#            set hunt_hits2 0
#            set hunt_hits3 $huntinfo(hits_remaining)
#            foreach i $result {
#              if {($i >= $${NamespaceParent}::wrpgdice::diceconf(low1_1)) && \
#                  ($i <= $${NamespaceParent}::wrpgdice::diceconf(low1_2))} then {
#                incr hunt_hits1 2
#              } elseif {($i >= $${NamespaceParent}::wrpgdice::diceconf(low2_1)) && \
#                        ($i <= $${NamespaceParent}::wrpgdice::diceconf(low2_2))} then {
#                incr hunt_hits1
#              } elseif {($i >= $${NamespaceParent}::wrpgdice::diceconf(high1_1)) && \
#                        ($i <= $${NamespaceParent}::wrpgdice::diceconf(high1_2))} then {
#                incr hunt_hits2
#                incr huntinfo(hits_scored)
#                incr huntinfo(hits_remaining) -1
#              } elseif {($i >= $${NamespaceParent}::wrpgdice::diceconf(high2_1)) && \
#                        ($i <= $${NamespaceParent}::wrpgdice::diceconf(high2_2))} then {
#                incr hunt_hits2 2
#                incr huntinfo(hits_scored) 2
#                incr huntinfo(hits_remaining) -2
#              }
#            }
#            foreach i $result {
#              lappend reply "\002$i\002,"
#            }
#            if {$hunt_hits1 > 0} then {
#              if {[hp decr $hand $hunt_hits1]} then {
#                lappend reply "you took [grammar_check1 0 $hunt_hits1 point] of damage,"
#              }
#            }
#            if {$hunt_hits2 > 0} then {
#              if {$huntinfo(hits_remaining) > 0} then {
#                lappend reply "the [join $huntinfo(animal)] recieved [grammar_check1 0 $hunt_hits2 point] of damage,"
#              } else {
#                lappend reply "the [join $huntinfo(animal)] recieved [grammar_check1 0 $hunt_hits3 point] of damage,"
#              }
#            }
#            if {$huntinfo(hits_remaining) > 0} then {
#              lappend reply "the [join $huntinfo(animal)] has [grammar_check1 0 $huntinfo(hits_remaining) hit] left out of $huntinfo(hits_total),"
#            } else {
#              if {$conf(autocaught)} then {
#                lappend reply "the [join $huntinfo(animal)] has been caught!"
#              } else {
#                lappend reply "the [join $huntinfo(animal)] has been killed!"
#              }
#              if {[lindex $huntdata($huntinfo(animal)) 4]} then {
#                personality huntkill "%nick $nick" "%chan $chan" "%animal $huntinfo(animal)"
#              }
#              set huntinfo(caught) 1
#              hunt_catch $huntinfo(animal)
#            }
#            if {[info exists reply]} then {
#              putmsg $chan "$nick rolls a [string trimright [join $reply] ,]"
#              return 1
#            }
#          } else {
#            putnot $nick "Your level is not high enough to end a hunt for [grammar_check2 0 [join $huntinfo(animal)]]"
#            return 1
#          }
#        } else {
#          putmsg $chan "Sorry $nick, you are too weak to hunt."
#          return 1
#        }
#      }
#    }
#    return 0
  }

  # clear all the info from a hunt
  proc clear_hunt {} {
#    variable huntinfo
#
#    if {$huntinfo(diceopt)} then {
#      config_dice reset ""
#    }
#    set huntinfo(animal) ""
#    set huntinfo(active) 0
#    set huntinfo(caught) 0
#    set huntinfo(channel) ""
#    set huntinfo(time) 0
#    set huntinfo(diceopt) 0
#    if {[info exists huntinfo(hits_total)]} then {unset huntinfo(hits_total)}
#    if {[info exists huntinfo(hits_remaining)]} then {unset huntinfo(hits_remaining)}
#    if {[info exists huntinfo(hits_scored)]} then {unset huntinfo(hits_scored)}
#    return 0
  }

  # check for and timeout an inactive hunt once a minute
  proc time:minutely {min hour day month year} {
#    variable conf
#    variable huntinfo
#
#    if {$huntinfo(active)} then {
#      if {$huntinfo(time)} then {
#        if {[expr ([clock seconds] - $huntinfo(time)) / 60] >= $conf(timeout)} then {
#          if {[validchan $huntinfo(channel)]} then {
#            putmsg $huntinfo(channel) "Hunt for [grammar_check2 0 [join $huntinfo(animal)]] timed out after [grammar_check1 0 $conf(timeout) minute], [grammar_check3 [grammar_check1 0 $huntinfo(hits_scored) hit]] scored out of $huntinfo(hits_total)."
#          }
#          clear_hunt
#        }
#      }
#    }
#    return 0
  }

  # Configuration data defaults
  array set configDataDefaults {
    allownonpack 1
    autocaught 0
    foodbury 0
    timeout 10
  }

  # Configuration data descriptions
  array set configDataDesc {
    allownonpack "\[0/1\] allow non pack to participate in a hunt?"
    autocaught "\[0/1\] automatically register a catch"
    foodbury "\[0/1\] remove animals from the food list automatically."
    timeout "\[minutes\] minutes before a hunt times out"
  }

  ##
  ## Init module specific things at load time
  ##
  ## Args:    action {load|unload}
  ## Returns: nothing
  ##
  proc InitTable {action} {
    variable huntInfo
    variable huntInfoDefaults

    switch -exact -- $action {
      load {
        foreach {name data} [array get huntInfoDefaults] {
          if {![info exists huntInfo($name)]} then {
            set huntInfo($name) $data
          }
        }
      }
    }
    return
  }

  # Default settings
  array set huntInfoDefaults {
    active 0
    animal ""
    caught 0
    channel ""
    diceopt ""
    time 0
  }

  ##
  ## Remove spoiled food once an hour
  ##
  ## Args:    minute, hour, day, month, year
  ## Returns: 0
  ##
  proc time:hourly {min hour day month year} {
    variable configData
    variable fooddata
    variable fooddataChanged

    if {$configData(foodbury)} then {
      foreach {name data} [array get fooddata] {
        foreach {amount timestamp} $data {break}
        if {($amount <= 0) || \
            ([daysLeft $timestamp $configData(foodspoil)] <= 0)} then {
          unset fooddata($name)
          set fooddataChanged 1
        }
      }
    }
    return 0
  }


###############################################################################



# diceConfig use ${NamespaceCurrent}::callback_proc
# proc callback_proc {nick uhost hand chan results} {
#   foreach i $results { ... }
#   return 1
# }


  # sniff out prey; pub
  proc pub:sniff {nick uhost hand chan arg} {
    variable huntdata
    variable huntinfo
    variable NamespaceCurrent

    if {[validHandle $hand} then {
      if {[hp test1 $hand 0] != 2} then {
        if {!$huntinfo(active)} then {
          set arg [split [string trimright $arg]]
          if {![string length $arg]} then {
            set animal [random_animal]
            if {[string length $animal]} then {
              set huntinfo(animal) $animal
              set huntinfo(time) [clock seconds]
              putmsg $chan "$nick catches the scent of [grammar_check2 0 [join $animal]] (It has [grammar_check1 0 [lindex $huntdata($animal) 1] hit])"
            } else {
              putnot $nick "No animals are in the hunt list."
            }
          } elseif {[isNumber $arg]} then {
            set animal [random_animal $arg]
            if {[string length $animal]} then {
              set huntinfo(animal) $animal
              set huntinfo(time) [clock seconds]
              putmsg $chan "$nick catches the scent of [grammar_check2 0 [join $animal]] (It has [grammar_check1 0 [lindex $huntdata($animal) 1] hit])"
            } else {
              putnot $nick "No animals are in the hunt list for group $arg"
            }
          } else {
            set animal [arrayFindElementName ${NamespaceCurrent}::huntdata $arg]
            if {[string length $animal]} then {
              set huntinfo(animal) $animal
              set huntinfo(time) [clock seconds]
              putmsg $chan "$nick catches the scent of [grammar_check2 0 [join $animal]] (It has [grammar_check1 0 [lindex $huntdata($animal) 1] hit])"
            } else {
              putnot $nick "I don't know what [grammar_check2 0 $arg] is."
            }
          }
        } else {
          putnot $nick "A hunt for [grammar_check2 0 [join $huntinfo(animal)]] is already in progress."
        }
      } else {
        putmsg $chan "Sorry $nick, you are too weak to hunt."
      }
      return 1
    }
    return 0
  }

  # start a hunt
  proc pub:hunt {nick uhost hand chan arg} {
    variable huntdata
    variable huntinfo
    variable NamespaceCurrent

    if {[validHandle $hand]} then {
      if {[hp test1 $hand 0] != 2} then {
        if {!$huntinfo(active)} then {
          set arg [split [string trimright $arg]]
          if {[string length $arg]} then {
            set animal [arrayFindElementName ${NamespaceCurrent}::huntdata $arg]
          } elseif {[string length $huntinfo(animal)]} then {
            set animal $huntinfo(animal)
          }
          if {[info exists animal]} then {
            if {[string length $animal]} then {
              if {[lev list $hand ""] >= [lindex $huntdata($animal) 0]} then {
                config_dice use ${NamespaceCurrent}::hunt_dice
                set diceopt [lindex $huntdata($animal) 5]
                if {[string length $diceopt]} then {
                  config_dice configure $diceopt
                  set huntinfo(diceopt) 1
                }
                set huntinfo(active) 1
                set huntinfo(caught) 0
                set huntinfo(channel) $chan
                set huntinfo(animal) $animal
                set huntinfo(time) [clock seconds]
                set huntinfo(hits_total) [lindex $huntdata($animal) 1]
                set huntinfo(hits_remaining) $huntinfo(hits_total)
                set huntinfo(hits_scored) 0
                putmsg $chan "Hunt for [grammar_check2 0 [join $animal]] now active! (It has [grammar_check1 0 $huntinfo(hits_total) hit])"
              } else {
                putnot $nick "Your level is not high enough to start a hunt for [grammar_check2 0 [join $animal]]"
              }
            } else {
              putnot $nick "I don't know what [grammar_check2 0 $arg] is."
            }
          } else {
            putnot $nick "You must sniff out an animal or specify what to hunt."
          }
        } else {
          putnot $nick "A hunt for [grammar_check2 0 [join $huntinfo(animal)]] is already in progress."
        }
      } else {
        putmsg $chan "Sorry $nick, you are too weak to hunt."
      }
      return 1
    }
    return 0
  }

  #FIXME: $huntinfo(channel)
  # immediately terminate a hunt
  proc pub:endhunt {nick uhost hand chan arg} {
    variable huntdata
    variable huntinfo
    variable NamespaceCurrent

    if {[validHandle $hand]} then {
      if {[hp test1 $hand 0] != 2} then {
        if {$huntinfo(active)} then {
          set arg [split [string trimright $arg]]
          if {[string length $arg]} then {
            set animal [arrayFindElementName ${NamespaceCurrent}::huntdata $arg]
          } else {
            set animal $huntinfo(animal)
          }
          if {[string length $animal]} then {
            if {[lev list $hand ""] >= [lindex $huntdata($animal) 0]} then {
              if {![string compare $animal $huntinfo(animal)]} then {
                putmsg $chan "Hunt for [grammar_check2 0 [join $huntinfo(animal)]] ended, [grammar_check3 [grammar_check1 0 $huntinfo(hits_scored) hit]] scored out of $huntinfo(hits_total)."
                clear_hunt
              } else {
                putnot $nick "[grammar_check2 1 [join $animal]] is not being hunted, [grammar_check2 0 [join $huntinfo(animal)]] is."
              }
            } else {
              putnot $nick "Your level is not high enough to end a hunt for [grammar_check2 0 [join $animal]]"
            }
          } else {
            putnot $nick "I don't know what [grammar_check2 0 $arg] is."
          }
        } else {
          putnot $nick "There wasn't a hunt active."
        }
      } else {
        putnot $chan "Sorry $nick, you are too weak to hunt."
      }
      return 1
    }
    return 0
  }

  # register caught prey from the hunt; pub
  proc pub:caught {nick uhost hand chan arg} {
    variable conf
    variable huntinfo
    variable huntdata
    variable fooddata
    variable NamespaceCurrent

    if {[validHandle $hand]} then {
      if {[hp test1 $hand 0] != 2} then {
        set arg [split [string trimright $arg]]
        if {[string length $arg]} then {
          set animal [arrayFindElementName ${NamespaceCurrent}::huntdata $arg]
        } elseif {$huntinfo(active)} then {
          set animal $huntinfo(animal)
        }
        if {[info exists animal]} then {
          if {[string length $animal]} then {
            if {$huntinfo(active)} then {
              if {!$conf(autocaught)} then {
                if {![string compare $animal $huntinfo(animal)]} then {
                  if {[hunt_catch $animal]} then {
                    putmsg $chan "There is now enough [join $animal] for [lindex $fooddata($animal) 0] wolves."
                  } else {
                    putnot $nick "The [join $animal] hasn't been caught yet!"
                  }
                } else {
                  putnot $nick "[grammar_check2 1 [join $animal]] was not caught, [grammar_check2 0 [join $huntinfo(animal)]] was."
                }
              } else {
                putnot $nick "The caught command is unnecessary since autocaught is enabled."
              }
            } elseif {[lindex $huntdata($animal) 3]} then {
              if {[hunt_catch $animal]} then {
                putmsg $chan "There is now enough [join $animal] for [lindex $fooddata($animal) 0] wolves."
              }
            } else {
              putnot $nick "There isn't a hunt active."
            }
          } else {
            putnot $nick "I don't know what [grammar_check2 0 [join $animal]] is."
          }
        } else {
          putnot $nick "You must hunt an animal or specify what to catch."
        }
      } else {
        putmsg $chan "Sorry $nick, you are too weak to hunt."
      }
      return 1
    }
    return 0
  }

  # register caught prey from the hunt
  proc hunt_catch {animal} {
    variable huntinfo
    variable huntdata
    variable fooddata
    variable fooddata_changed

    if {($huntinfo(caught)) || (![lindex $huntdata($animal) 3])} then {
      if {![info exists fooddata($animal)]} then {
        set fooddata($animal) "[lindex $huntdata($animal) 2] [clock seconds]"
      } else {
        set fooddata($animal) "[expr [lindex $fooddata($animal) 0] + [lindex $huntdata($animal) 2]] [clock seconds]"
      }
      set fooddata_changed 1
      if {$huntinfo(active)} then {
        clear_hunt
      }
      return 1
    }
    return 0
  }


  # eat prey to increase user's fp
  proc pub:eat {nick uhost hand chan arg} {
    variable conf
    variable fooddata
    variable fooddata_changed
    variable NamespaceCurrent

    if {[validHandle $hand]} then {
      set arg [split [string trimright $arg]]
      if {![string length $arg]} then {
        if {$conf(foodspoil)} then {
          foreach i [lsort [array names fooddata]] {
            lappend list "[join $i] \[[lindex $fooddata($i) 0]:[days_left [lindex $fooddata($i) 1] $conf(foodspoil)]\],"
          }
        } else {
          foreach i [lsort [array names fooddata]] {
            lappend list "[join $i] \[[lindex $fooddata($i) 0]\],"
          }
        }
        if {[info exists list]} then {
          formatLists putmsg $chan [string trimright [join $list] ,] 160 , , " " "The food available is: " "(cont): "
        } else {
          putmsg $chan "The food list is empty :("
        }
      } else {
        if {[isNumber [lindex $arg end]]} then {
          set animal [arrayFindElementName ${NamespaceCurrent}::huntdata [lrange $arg 0 [expr [llength $arg] - 2]]]
          set amount [lindex $arg end]
        } else {
          set animal [arrayFindElementName ${NamespaceCurrent}::huntdata $arg]
          set amount 1
        }
        #FIXME: $animal == ""
        if {[info exists fooddata($animal)]} then {
          set count [lindex $fooddata($animal) 0]
          if {$count > 0} then {
            if {([days_left [lindex $fooddata($animal) 1] $conf(foodspoil)] > 0) || (!$conf(foodspoil))} then {
              if {![matchattr $hand F]} then {
                if {[fp test2 $hand 0] != 2} then {
                  if {[expr ([fp limit $hand 0] - [fp list $hand 0]) >= $amount]} then {
                    if {[isNumber $amount]} then {
                      if {[lindex $fooddata($animal) 0] >= $amount} then {
                        #FIXME: need to do this all at once
                        for {set count2 $amount} {$count2 > 0} {incr count2 -1} {
                          if {$count > 1} then {
                            if {[fp incr $hand 1]} then {
                              set fooddata($animal) [lreplace $fooddata($animal) 0 0 [expr [lindex $fooddata($animal) 0] - 1]]
                              set fooddata_changed 1
                              incr count -1
                            }
                          } else {
                            if {[fp incr $hand 1]} then {
                              set fooddata($animal) [lreplace $fooddata($animal) 0 0 0]
                              set fooddata_changed 1
                              incr count -1
                            }
                          }
                        }
                        if {$count < 1} then {
                          if {$conf(foodbury)} then {
                            unset fooddata($animal)
                          }
                          putmsg $chan "$nick eats the last of the [join $animal]."
                        } elseif {$count == 1} then {
                          putmsg $chan "$nick eats some of the [join $animal], there is only enough left for one more wolf."
                        } elseif {$count > 1} then {
                          putmsg $chan "$nick eats some of the [join $animal], there is enough left for [lindex $fooddata($animal) 0] more wolves."
                        }
                      } else {
                        putnot $nick "There is not that much [join $animal] to eat."
                      }
                    } else {
                      putnot $nick "Invalid count of [join $animal] to eat."
                    }
                  } else {
                    putmsg $chan "Sorry $nick, but you are too full to eat that much [join $animal]."
                  }
                } else {
                  putmsg $chan "Sorry $nick, but you are too full to eat."
                }
              } else {
                putnot $nick "You don't need to eat anything."
              }
            } else {
              putmsg $chan "The [join $animal] has spoiled, please bury it."
            }
          } else {
            putmsg $chan "There is nothing but a carcass left of the [join $animal], please bury it."
          }
        } else {
          putmsg $chan "There is no [join $animal] to eat."
        }
      }
      return 1
    }
    return 0
  }




  # bury prey that has been eaten or has spoiled
  proc pub:bury {nick uhost hand chan arg} {
    variable conf
    variable fooddata
    variable fooddata_changed
    variable NamespaceCurrent

    if {[validHandle $hand]} then {
      set arg [split [string trimright $arg]]
      if {[string length $arg]} then {
        set animal [arrayFindElementName ${NamespaceCurrent}::huntdata $arg]
        if {[string length $animal]} then {
          if {[info exists fooddata($animal)]} then {
            if {([lindex $fooddata($animal) 0] <= 0) || \
                (!$conf(foodspoil)) || \
                ([days_left [lindex $fooddata($animal) 1] $conf(foodspoil)] <= 0)} then {
              putmsg $chan "$nick buries the [join $animal]"
              unset fooddata($animal)
              set fooddata_changed 1
            } else {
              putnot $nick "There is still [join $animal] left."
            }
          } else {
            putnot $nick "Theres no [join $animal] to bury."
          }
        } else {
          putnot $nick "I don't know what [grammar_check2 0 $arg] is."
        }
      } else {
        putnot $nick "You must specify the animal to bury."
      }
      return 1
    }
    return 0
  }

  # score a hunt
  proc hunt_dice {nick uhost hand chan result} {
    variable conf
    variable huntinfo
    variable huntdata
    variable NamespaceParent

    if {$huntinfo(active)} then {
      if {!$huntinfo(caught)} then {
        if {[hp test1 $hand 0] != 2} then {
          if {(($conf(allownonpack)) && (![validuser $hand])) || ([lev list $hand 0])} then {
            set huntinfo(time) [clock seconds]
            set hunt_hits1 0
            set hunt_hits2 0
            set hunt_hits3 $huntinfo(hits_remaining)
            foreach i $result {
              if {($i >= $${NamespaceParent}::wrpgdice::diceconf(low1_1)) && \
                  ($i <= $${NamespaceParent}::wrpgdice::diceconf(low1_2))} then {
                incr hunt_hits1 2
              } elseif {($i >= $${NamespaceParent}::wrpgdice::diceconf(low2_1)) && \
                        ($i <= $${NamespaceParent}::wrpgdice::diceconf(low2_2))} then {
                incr hunt_hits1
              } elseif {($i >= $${NamespaceParent}::wrpgdice::diceconf(high1_1)) && \
                        ($i <= $${NamespaceParent}::wrpgdice::diceconf(high1_2))} then {
                incr hunt_hits2
                incr huntinfo(hits_scored)
                incr huntinfo(hits_remaining) -1
              } elseif {($i >= $${NamespaceParent}::wrpgdice::diceconf(high2_1)) && \
                        ($i <= $${NamespaceParent}::wrpgdice::diceconf(high2_2))} then {
                incr hunt_hits2 2
                incr huntinfo(hits_scored) 2
                incr huntinfo(hits_remaining) -2
              }
            }
            foreach i $result {
              lappend reply "\002$i\002,"
            }
            if {$hunt_hits1 > 0} then {
              if {[hp decr $hand $hunt_hits1]} then {
                lappend reply "you took [grammar_check1 0 $hunt_hits1 point] of damage,"
              }
            }
            if {$hunt_hits2 > 0} then {
              if {$huntinfo(hits_remaining) > 0} then {
                lappend reply "the [join $huntinfo(animal)] recieved [grammar_check1 0 $hunt_hits2 point] of damage,"
              } else {
                lappend reply "the [join $huntinfo(animal)] recieved [grammar_check1 0 $hunt_hits3 point] of damage,"
              }
            }
            if {$huntinfo(hits_remaining) > 0} then {
              lappend reply "the [join $huntinfo(animal)] has [grammar_check1 0 $huntinfo(hits_remaining) hit] left out of $huntinfo(hits_total),"
            } else {
              if {$conf(autocaught)} then {
                lappend reply "the [join $huntinfo(animal)] has been caught!"
              } else {
                lappend reply "the [join $huntinfo(animal)] has been killed!"
              }
              if {[lindex $huntdata($huntinfo(animal)) 4]} then {
                personality huntkill "%nick $nick" "%chan $chan" "%animal $huntinfo(animal)"
              }
              set huntinfo(caught) 1
              hunt_catch $huntinfo(animal)
            }
            if {[info exists reply]} then {
              putmsg $chan "$nick rolls a [string trimright [join $reply] ,]"
              return 1
            }
          } else {
            putnot $nick "Your level is not high enough to end a hunt for [grammar_check2 0 [join $huntinfo(animal)]]"
            return 1
          }
        } else {
          putmsg $chan "Sorry $nick, you are too weak to hunt."
          return 1
        }
      }
    }
    return 0
  }

  # clear all the info from a hunt
  proc clear_hunt {} {
    variable huntinfo

    if {$huntinfo(diceopt)} then {
      config_dice reset ""
    }
    set huntinfo(animal) ""
    set huntinfo(active) 0
    set huntinfo(caught) 0
    set huntinfo(channel) ""
    set huntinfo(time) 0
    set huntinfo(diceopt) 0
    if {[info exists huntinfo(hits_total)]} then {unset huntinfo(hits_total)}
    if {[info exists huntinfo(hits_remaining)]} then {unset huntinfo(hits_remaining)}
    if {[info exists huntinfo(hits_scored)]} then {unset huntinfo(hits_scored)}
    return 0
  }

  # check for and timeout an inactive hunt once a minute
  proc time:minutely {min hour day month year} {
    variable conf
    variable huntinfo

    if {$huntinfo(active)} then {
      if {$huntinfo(time)} then {
        if {[expr ([clock seconds] - $huntinfo(time)) / 60] >= $conf(timeout)} then {
          if {[validchan $huntinfo(channel)]} then {
            putmsg $huntinfo(channel) "Hunt for [grammar_check2 0 [join $huntinfo(animal)]] timed out after [grammar_check1 0 $conf(timeout) minute], [grammar_check3 [grammar_check1 0 $huntinfo(hits_scored) hit]] scored out of $huntinfo(hits_total)."
          }
          clear_hunt
        }
      }
    }
    return 0
  }

  # Configuration data defaults
  array set configDataDefaults {
    allownonpack 1
    autocaught 0
    foodbury 0
    timeout 10
  }

  # Configuration data descriptions
  array set configDataDesc {
    allownonpack "\[0/1\] allow non pack to participate in a hunt?"
    autocaught "\[0/1\] automatically register a catch"
    foodbury "\[0/1\] remove animals from the food list automatically."
    timeout "\[minutes\] minutes before a hunt times out"
  }

  ##
  ## Init module specific things at load time
  ##
  ## Args:    action {load|unload}
  ## Returns: nothing
  ##
  proc InitTable {action} {
    variable huntInfo
    variable huntInfoDefaults

    switch -exact -- $action {
      load {
        foreach {name data} [array get huntInfoDefaults] {
          if {![info exists huntInfo($name)]} then {
            set huntInfo($name) $data
          }
        }
      }
    }
    return
  }

  # Default settings
  array set huntInfoDefaults {
    active 0
    animal ""
    caught 0
    channel ""
    diceopt ""
    time 0
  }

  ##
  ## Remove spoiled food once an hour
  ##
  ## Args:    minute, hour, day, month, year
  ## Returns: 0
  ##
  proc time:hourly {min hour day month year} {
    variable configData
    variable fooddata
    variable fooddataChanged

    if {$configData(foodbury)} then {
      foreach {name data} [array get fooddata] {
        foreach {amount timestamp} $data {break}
        if {($amount <= 0) || \
            ([daysLeft $timestamp $configData(foodspoil)] <= 0)} then {
          unset fooddata($name)
          set fooddataChanged 1
        }
      }
    }
    return 0
  }


############################################################


  # adds an animal to the hunt list; pub
  proc pub:addhunt {nick uhost hand chan arg} {
    return [addhunt pub putnot $nick putnot $nick $nick $uhost $hand $chan $arg]
  }

  # adds an animal to the hunt list; msg
  proc msg:addhunt {nick uhost hand arg} {
    return [addhunt msg putnot $nick putnot $nick $nick $uhost $hand "" $arg]
  }

  # adds an animal to the hunt list; dcc
  proc dcc:addhunt {hand idx arg} {
    return [addhunt dcc putdcc $idx putdcc $idx "" "" $hand "" $arg]
  }

  # adds an animal to the hunt list
  proc addhunt {type resp1 dest1 resp2 dest2 nick uhost hand chan arg} {
    variable huntdata
    variable huntdata_changed
    variable NamespaceCurrent

    if {[authCheck $type addhunt $nick $uhost $hand $chan]} then {
      set arg [split [string trimright $arg]]
      set count [llength [split [lindex $arg end] :]]
      if {$count > 5} then {
        set animal [lrange $arg 0 [expr [llength $arg] - 3]]
        set options [lindex $arg [expr [llength $arg] - 2]]
        set diceopt [lindex $arg end]
      } else {
        set animal [lrange $arg 0 [expr [llength $arg] - 2]]
        set options [lindex $arg end]
      }
      scan $options "%\[^:\]:%\[^:\]:%\[^:\]:%\[^:\]:%\[^:\]:" group hits food hunt sound
      if {(![string length $animal]) || (![info exists group]) || (![info exists hits]) || (![info exists food]) || (![info exists hunt]) || (![info exists sound])} then {
        put $resp2 $dest2 "Usage: [command_char $type]addhunt <animal> <group:hits:food:hunt:sound> \[diceopt\]"
      } else {
        if {[info exists diceopt]} then {
          regsub -all -- : $diceopt " " diceopt
          regsub -all -- - $diceopt " " diceopt
        } else {
          set diceopt ""
        }
        set animal [arrayFindElementName ${NamespaceCurrent}::huntdata $animal]
        if {![string length $animal]} then {
          if {![isNumber [lindex $animal end]]} then {
            if {[isNumber $group]} then {
              if {[isNumber $hits]} then {
                if {[isNumber $food]} then {
                  if {[isNumber $hunt]} then {
                    if {[isNumber $sound]} then {
                      set huntdata($animal) [list $group $hits $food $hunt $sound $diceopt]
                      set huntdata_changed 1
                      put $resp1 $dest1 "The [join $animal] has been added to the hunt list."
                    } else {
                      put $resp2 $dest2 "Invalid number to set sound to."
                    }
                  } else {
                    put $resp2 $dest2 "Invalid humber to set hunt to."
                  }
                } else {
                  put $resp2 $dest2 "Invalid number to set food to."
                }
              } else {
                put $resp2 $dest2 "Invalid number to set hits to."
              }
            } else {
              put $resp2 $dest2 "Invalid number to set group to."
            }
          } else {
            put $resp2 $dest2 "Animal's name may not end with a number"
          }
        } else {
          put $resp2 $dest2 "I already know what [grammar_check2 0 [join $animal]] is."
        }
      }
      return 1
    }
    return 0
  }

  # removes an animal to the hunt list; pub
  proc pub:delhunt {nick uhost hand chan arg} {
    return [delhunt pub putnot $nick putnot $nick $nick $uhost $hand $chan $arg]
  }

  # removes an animal to the hunt list; msg
  proc msg:delhunt {nick uhost hand arg} {
    return [delhunt msg putnot $nick putnot $nick $nick $uhost $hand "" $arg]
  }

  # removes an animal to the hunt list; dcc
  proc dcc:delhunt {hand idx arg} {
    return [delhunt dcc putdcc $idx putdcc $idx "" "" $hand "" $arg]
  }

  # removes an animal to the hunt list
  proc delhunt {type resp1 dest1 resp2 dest2 nick uhost hand chan arg} {
    variable huntdata
    variable fooddata
    variable huntdata_changed
    variable fooddata_changed
    variable NamespaceCurrent

    if {[authCheck $type delhunt $nick $uhost $hand $chan]} then {
      set arg [split [string trimright $arg]]
      if {![string length $arg]} then {
        put $resp2 $dest2 "Usage: [command_char $type]delhunt <animal>"
      } else {
        set animal [arrayFindElementName ${NamespaceCurrent}::huntdata $arg]
        if {[string length $animal]} then {
          if {[info exists fooddata($animal)]} then {
            unset fooddata($animal)
            set fooddata_changed 1
            unset huntdata($animal)
            set huntdata_changed 1
            put $resp1 $dest1 "The [join $animal] has been removed from the hunt and food lists."
          } else {
            unset huntdata($animal)
            set huntdata_changed 1
            put $resp1 $dest1 "The [join $animal] has been removed from the hunt list."
          }
        } else {
          put $resp2 $dest2 "I don't know what [grammar_check2 0 [join $animal]] is."
        }
      }
      return 1
    }
    return 0
  }

  # changes an animal to the hunt list; pub
  proc pub:chhunt {nick uhost hand chan arg} {
    return [chhunt pub putnot $nick putnot $nick $nick $uhost $hand $chan $arg]
  }

  # changes an animal to the hunt list; msg
  proc msg:chhunt {nick uhost hand arg} {
    return [chhunt msg putnot $nick putnot $nick $nick $uhost $hand "" $arg]
  }

  # changes an animal to the hunt list; dcc
  proc dcc:chhunt {hand idx arg} {
    return [chhunt dcc putdcc $idx putdcc $idx "" "" $hand "" $arg]
  }

  # changes an animal to the hunt list
  proc chhunt {type resp1 dest1 resp2 dest2 nick uhost hand chan arg} {
    variable huntdata
    variable fooddata
    variable huntdata_changed
    variable fooddata_changed
    variable NamespaceCurrent

    if {[authCheck $type chhunt $nick $uhost $hand $chan]} then {
      set arg [split [string trimright $arg]]
      set count [llength [split [lindex $arg end] :]]
      if {$count > 5} then {
        set animal [arrayFindElementName ${NamespaceCurrent}::huntdata [lrange $arg 0 [expr [llength $arg] - 3]]]
        set options [lindex $arg [expr [llength $arg] - 2]]
        set diceopt [lindex $arg end]
        set tmp [split $options :]
        scan $options "%\[^:\]:%\[^:\]:%\[^:\]:%\[^:\]:%\[^:\]:" group hits food hunt sound
      } elseif {$count > 2} then {
        set animal [arrayFindElementName ${NamespaceCurrent}::huntdata [lrange $arg 0 [expr [llength $arg] - 2]]]
        set options [lindex $arg end]
        set tmp [split $options :]
        scan $options "%\[^:\]:%\[^:\]:%\[^:\]:%\[^:\]:%\[^:\]:" group hits food hunt sound
      } else {
        set animal [arrayFindElementName ${NamespaceCurrent}::huntdata [string trimright [lindex [split $arg :] 0]]]
        set animal2 [string trimleft [lindex [split $arg :] 1]]
      }
      if {![string length $arg]} then {
        put $resp2 $dest2 "Usage: [command_char $type]chhunt <animal> <group:hits:food:hunt:sound> \[diceopt\] | <oldanimal>:<newanimal>"
      } elseif {[string length $animal]} then {
        if {[info exists animal2]} then {
          if {![string compare $animal $animal2]} then {
            if {![isNumber [lindex $animal end]]} then {
              if {[info exists fooddata($animal)]} then {
                set fooddata($animal2) $fooddata($animal)
                unset fooddata($animal)
                set fooddata_changed 1
              }
              set huntdata($animal2) $huntdata($animal)
              unset huntdata($animal)
              set huntdata_changed 1
              put $resp1 $dest1 "Renamed [join $animal] to [join $animal2]."
            } else {
              put $resp2 $dest2 "Animal name may not end with a number"
            }
          } else {
            put $resp2 $dest2 "You must specify a different name to rename an animal to."
          }
        } else {
          if {[info exists diceopt]} then {
            regsub -all -- : $diceopt " " diceopt
            regsub -all -- - $diceopt " " diceopt
          } else {
            set diceopt [lindex $huntdata($animal) 5]
          }
          if {[isNumber $group]} then {
            if {[isNumber $hits]} then {
              if {[isNumber $food]} then {
                if {[isNumber $hunt]} then {
                  if {[isNumber $sound]} then {
                    #FIXME: if provided, show dice options when adding
                    if {([lrange $huntdata($animal) 0 4] != $tmp) || ([lindex $huntdata($animal) 5] != $diceopt)} then {
                      set huntdata($animal) [list $group $hits $food $hunt $sound $diceopt]
                      set huntdata_changed 1
                      put $resp1 $dest1 "The [join $animal] has been set to: $group:$hits:$food:$hunt:$sound"
                    } else {
                      put $resp2 $dest2 "The [join $animal] was already set to: $group:$hits:$food:$hunt:$sound"
                    }
                  } else {
                    put $resp2 $dest2 "Invalid number to set sound to."
                  }
                } else {
                  put $resp2 $dest2 "Invalid humber to set hunt to."
                }
              } else {
                put $resp2 $dest2 "Invalid number to set food to."
              }
            } else {
              put $resp2 $dest2 "Invalid number to set hits to."
            }
          } else {
            put $resp2 $dest2 "Invalid number to set group to."
          }
        }
      } else {
        put $resp2 $dest2 "I don't know what [grammar_check2 0 [join $animal]] is."
      }
      return 1
    }
    return 0
  }

  # lists the hunt list; pub
  proc pub:listhunt {nick uhost hand chan arg} {
    return [listhunt pub putnot $nick putnot $nick $nick $uhost $hand $chan $arg]
  }

  # lists the hunt list; msg
  proc msg:listhunt {nick uhost hand arg} {
    return [listhunt msg putnot $nick putnot $nick $nick $uhost $hand "" $arg]
  }

  # lists the hunt list; dcc
  proc dcc:listhunt {hand idx arg} {
    return [listhunt dcc putdcc $idx putdcc $idx "" "" $hand "" $arg]
  }

  # lists the hunt list
  proc listhunt {type resp1 dest1 resp2 dest2 nick uhost hand chan arg} {
    variable huntdata
    variable NamespaceCurrent

    if {[authCheck $type listhunt $nick $uhost $hand $chan]} then {
      set what [split [string trimright $arg]]
      set count 0
      if {![string length $what]} then {
        foreach i [lsort [array names huntdata]] {
          lappend list "[join $i] \[[lindex $huntdata($i) 0]:[lindex $huntdata($i) 1]:[lindex $huntdata($i) 2]:[lindex $huntdata($i) 3]:[lindex $huntdata($i) 4]\],"
          incr count
        }
        if {[info exists list]} then {
          formatLists $resp1 $dest1 [string trimright [join $list] ,] 160 , , " " "The following animals are on the hunt list ($count): " "(cont): "
        } else {
          put $resp2 $dest2 "The hunt list is empty :("
        }
      } elseif {[isNumber $what]} then {
        foreach i [lsort [array names huntdata]] {
          if {![string compare $what [lindex $huntdata($i) 0]]} then {
            lappend list "[join $i] \[[lindex $huntdata($i) 0]:[lindex $huntdata($i) 1]:[lindex $huntdata($i) 2]:[lindex $huntdata($i) 3]:[lindex $huntdata($i) 4]\],"
            incr count
          }
        }
        if {[info exists list]} then {
          formatLists $resp1 $dest1 [string trimright [join $list] ,] 160 , , " " "The following animals are on the hunt list that are in group $what ($count): " "(cont): "
        } else {
          put $resp2 $dest2 "No animals in the hunt list for group $what"
        }
      } else {
        set animal [arrayFindElementName ${NamespaceCurrent}::huntdata $what]
        if {[string length $animal]} then {
          put $resp1 $dest1 "Info for [join $animal]: \[[lindex $huntdata($animal) 0]:[lindex $huntdata($animal) 1]:[lindex $huntdata($animal) 2]:[lindex $huntdata($animal) 3]:[lindex $huntdata($animal) 4]\]"
        } else {
          put $resp2 $dest2 "I don't know what [grammar_check2 0 $what] is."
        }
      }
      return 1
    }
    return 0
  }
