###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2001  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: statusdata.tcl,v 1.2 2001/06/01 05:38:49 tothwolf Exp $
##

# name: wrpg-statusdata
# version: 1.0.0.0
# config: wrpg-statusdata.conf
# description: Wolf RPG user status data support.
# requires: duration clearFields findField isNumber convertSeconds \
#	whoisFields wpLog
# provides: userStatusData userAssessmentData updateStatusData

namespace eval wrpg-statusdata {

  ##
  ## Handle a user's status/level/position data
  ##
  ## Args:    user, action {set|clear|compare|list|listusers|check}
  ##          what {status|level|pos}, data
  ## Returns:
  ##
  proc userStatusData {user action what args} {
    variable configData

    switch -exact $action {
      set {
        if {[string compare [string tolower [set data [lindex $args 0]]] \
             [string tolower [getuser $user XTRA $configData(${what}field)]]]} then {
          setuser $user XTRA [string toupper $configData(${what}field)] $data
          return 1
        }
      }
      clear {
        clearFields $user $configData(${what}field)
        return 1
      }
      compare {
        if {([string length [set data \
              [getuser $user XTRA $configData(${what}field)]]]) && \
             (![string compare [string tolower $data] \
                [string tolower [lindex $args 0]]])} then {
          return 1
        }
      }
      list {
        return [getuser $user XTRA $configData(${what}field)]
      }
      listusers {
        set list ""
        if {[string compare "" [set data [string tolower [lindex $args 0]]]]} then {
          foreach user [lsort [findField $configData(${what}field)]] {
            if {![string compare $data \
                  [string tolower \
                   [getuser $user XTRA $configData(${what}field)]]]} then {
              lappend list $user
            }
          }
        } else {
          foreach user [lsort [findField $configData(${what}field)]] {
            lappend list $user
          }
        }
        return $list
      }
      check {
        if {[info exists configData(default${what})]} then {
          setuser $user XTRA [string toupper $configData(${what}field)] $configData(default${what})
        }
      }
    }
    return 0
  }

  ##
  ## Handle a user's assessment data
  ##
  ## Args:    user, action {add|del|started|time|timeleft|update}, data
  ## Returns:
  ##
  proc userAssessmentData {user action args} {
    variable configData

    switch -exact -- $action {
      add {
        set time [clock seconds]
        set seconds [convertSeconds from days $configData(asdays)]
        setuser $user XTRA $configData(astimefield) $time
        if {$configData(asdays)} then {
          set info "Assessment ends: [clock format [expr $time + $seconds] -format "%a %b %d %Y"], Time left: [duration $seconds]"
        } else {
          set info "Assessment started: [clock format $time -format "%a %b %d %Y"]"
        }
        setuser $user XTRA [string toupper $configData(asinfofield)] $info
        return 1
      }
      del {
        clearFields $user "$configData(astimefield) $configData(asinfofield)"
        return 1
      }
      started {
        if {[isNumber [set time [getuser $user XTRA $configData(astimefield)]]]} then {
          return [clock format $time -format "%a %b %d %Y"]
        }
        return
      }
      time {
        if {[isNumber [set time [getuser $user XTRA $configData(astimefield)]]]} then {
          return $time
        }
        return 0
      }
      timeleft {
        if {$configData(asdays)} then {
          set astime [getuser $user XTRA $configData(astimefield)]
          set time [clock seconds]
          set timeleft [expr ($astime + [convertSeconds from days $configData(asdays)]) - $time]
          if {$timeleft > 0} then {
            return $timeleft
          } else {
            return 0
          }
        } else {
          return -1
        }
      }
      update {
        if {$configData(asdays)} then {
          set astime [getuser $user XTRA $configData(astimefield)]
          set time [clock seconds]
          set endtime [expr ($astime + [convertSeconds from days $configData(asdays)])]
          set timeleft [expr $endtime - $time]
          if {$timeleft > 0} then {
            set info "Assessment ends: [clock format $endtime -format "%a %b %d %Y"], time left: [duration $timeleft]"
          } else {
            set info "Assessment ended: [clock format $endtime -format "%a %b %d %Y"]"
          }
          setuser $user XTRA [string toupper $configData(asinfofield)] $info
        }
        return 1
      }
    }
    return
  }

  ##
  ## Check/update assessment time and status
  ##
  ## Args:    none
  ## Returns: nothing
  ##
  proc updateStatusData {} {
    variable NamespaceCurrent
    variable configData

    if {($configData(asdays)) && \
        ([string compare "" \
          [set users [userStatusData "" listusers status assessment]]])} then {
      foreach user $users {
        if {($configData(autoupdate)) && \
            (![userAssessmentData $user timeleft])} then {
          if {[userStatusData $user set status pack]} then {
            userAssessmentData $user del
            userStatusData $user check pos
            userStatusData $user check level
            wpLog o * $NamespaceCurrent "Updated status for $user from assessment to pack."
          } else {
            wpLog o * $NamespaceCurrent "Error updating status for $user from assessment to pack."
          }
        } else {
          userAssessmentData $user update
        }
      }
    }
    return
  }

  ##
  ## Trace configuration changes
  ##
  ## Args:    name1, name2, operation
  ## Returns: nothing
  ##
  proc TraceConfigData {name1 name2 op} {
    variable configData
    variable oldConfigData

    if {[string compare $oldConfigData($name2) $configData($name2)]} then {
      switch -exact -- $name2 {
        statusfield -
        levelfield -
        posfield -
        asinfofield {
          moveFields $oldConfigData($name2) [string toupper $configData($name2)]
          whoisFields update $configData($name2) $oldConfigData($name2)
        }
      }
      set oldConfigData($name2) $configData($name2)
    }
    return
  }

  ##
  ## Trace and respond to variable changes
  ##
  ## Args:    action {variable|vdelete}
  ## Returns: nothing
  ##
  proc TraceTable {action} {
    variable NamespaceCurrent
    variable configData

    trace $action configData w ${NamespaceCurrent}::TraceConfigData
    return
  }

  ##
  ## Init module specific things at load time
  ##
  ## Args:    action {load|unload}
  ## Returns: nothing
  ##
  proc InitTable {action} {
    variable NamespaceCurrent
    variable configData
    variable oldConfigData

    switch -exact -- $action {
      load {
        if {![info exists NamespaceCurrent]} then {
          set NamespaceCurrent [namespace current]
        }
        foreach {name data} [array get configData] {
          if {![info exists oldConfigData($name)]} then {
            set oldConfigData($name) $data
          }
        }
        whoisFields update "$configData(statusfield) $configData(levelfield) $configData(posfield) $configData(asinfofield)" \
                           "$oldConfigData(statusfield) $oldConfigData(levelfield) $oldConfigData(posfield) $oldConfigData(asinfofield)"
      }
      unload {
        whoisFields remove "$configData(statusfield) $configData(levelfield) $configData(posfield) $configData(asinfofield)"
      }
    }
    return
  }

  # Configuration data defaults
  array set configDataDefaults {
    autoupdate 1
    asdays 30
    asinfofield wrpgasinfo
    astimefield wrpgastime
    defaultlevel 1
    defaultpos "not assigned"
    levelfield wrpglevel
    posfield wrpgpos
    statusfield wrpgstatus
  }

  # Configuration data descriptions
  array set configDataDesc {
    autoupdate "\[0/1\] automatically update a user's status from assessment to pack?"
    asdays "\[days\] how many days assessments will last (0 for open ended)"
    asinfofield "\[field\] field to store assessment info"
    astimefield "\[field\] field to store assessment time"
    defaultlevel "\[level\] default level for user"
    defaultpos "\[position\] default position for user"
    levelfield "\[field\] field to store status level"
    posfield "\[field\] field to store position info"
    statusfield "\[field\] field to store user status"
  }

} ;# namespace wrpg-statusdata

return
